using AuditManager.Data;
using AuditManager.Interfaces;
using AuditManager.Models;
using AuditManager.Services;
using AuditManager.Shared.Constants;
using AuditManager.Views;
using Plugin.Connectivity;
using Plugin.Connectivity.Abstractions;
using Plugin.DeviceInfo;
using Syncfusion.Licensing;
using Syncfusion.XForms;
using System;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using System.Resources;
using Xamarin.Forms;

namespace AuditManager
{
    public partial class App : Application
    {
        static SQLiteDatabase database;

        public static SQLiteDatabase Database
        {
            get
            {
                if (database == null)
                    database = new SQLiteDatabase(Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), Constants.DatabaseName));

                return database;
            }
        }

        public static INavigationService NavigationService { get; } = new ViewNavigationService();

        public static Stopwatch Stopwatch { get; } = new Stopwatch();
        public static TimeSpan TimeSpan { get; set; } = new TimeSpan();


        public App()
        {
            SyncfusionLicenseProvider.RegisterLicense("NTE4MTZAMzEzNjJlMzQyZTMwV2dYVXJ4NjRCUEJhc3R2Q0hXbW8vTkEveEhCNzhzZE81T0l2Yjd6cXFTcz0=");

            InitializeComponent();

            if (Device.RuntimePlatform == Device.iOS || Device.RuntimePlatform == Device.Android)
            {
                var ci = DependencyService.Get<ILocalize>().GetCurrentCultureInfo();
                Resx.AppResources.Culture = ci;
                PopupLayoutResourceManager.Manager = new ResourceManager("AuditManager.Resx.Syncfusion.SfPopupLayout.XForms", GetType().GetTypeInfo().Assembly);
                DependencyService.Get<ILocalize>().SetLocale(ci);
            }

            Settings.IsConnected = CrossConnectivity.Current.IsConnected;
            CrossConnectivity.Current.ConnectivityChanged += CurrentConnectivityChanged;

            if (string.IsNullOrEmpty(Settings.DeviceId))
            {
                Settings.DeviceId = $"{CrossDeviceInfo.Current.Id}";
                Settings.DeviceName = $"{CrossDeviceInfo.Current.DeviceName}";
            }

            NavigationService.Configure(nameof(LoginPage), typeof(LoginPage));
            NavigationService.Configure(nameof(SplashScreenPage), typeof(SplashScreenPage));
            NavigationService.Configure(nameof(AuditListPage), typeof(AuditListPage));
            NavigationService.Configure(nameof(CategoriePage), typeof(CategoriePage));
            NavigationService.Configure(nameof(QuestionarioPage), typeof(QuestionarioPage));
            //NavigationService.Configure("ConvalidaAuditPage", typeof(CategoriePage));
            //NavigationService.Configure("ListaLavoriPage", typeof(CategoriePage));
            //NavigationService.Configure("CompletaAuditPage", typeof(CategoriePage));

            MainPage = GetFirstPage();
        }

        private void CurrentConnectivityChanged(object sender, ConnectivityChangedEventArgs e)
        {
            Settings.IsConnected = CrossConnectivity.Current.IsConnected;
            MessagingCenter.Send((App)Current, Messages.ConnectionMessage.ToString());
        }

        public static Page GetFirstPage()
        {

            if (string.IsNullOrEmpty(Settings.User) && string.IsNullOrEmpty(Settings.Password))
                return ((ViewNavigationService)NavigationService).SetRootPage(nameof(LoginPage));
            else
            {
                if (Settings.IsConnected && Settings.IsOnline)
                    return ((ViewNavigationService)NavigationService).SetRootPage(nameof(SplashScreenPage));
                else
                    return ((ViewNavigationService)NavigationService).SetRootPage(nameof(AuditListPage));
            }
        }

        protected override async void OnStart()
        {
            await Database.CheckDatabaseStructureAsync();
        }

        protected override void OnSleep()
        {

        }

        protected override void OnResume()
        {

        }

        public static Page GetMainPage()
        {
            //return ((ViewNavigationService)NavigationService).SetRootPage("AuditListPage", new AuditListViewModel());
            return ((ViewNavigationService)NavigationService).SetRootPage(nameof(AuditListPage));
        }
    }
}