﻿using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.ViewModels;
using AuditManager.Views;
using Syncfusion.ListView.XForms;
using Syncfusion.SfChart.XForms;
using Syncfusion.XForms.ProgressBar;
using Xamarin.Forms;

namespace AuditManager.Behaviors
{
    public class CategorieBehavior : Behavior<ContentPage>
    {
        private readonly Audit Audit;
        private SfListView listView;
        private CategorieViewModel Modello;

        public CategorieBehavior(Audit item)
        {
            Audit = item;
        }

        protected override void OnAttachedTo(ContentPage bindable)
        {
            listView = bindable.FindByName<SfListView>("listView");
            Modello = new CategorieViewModel(Audit);

            bindable.BindingContext = Modello;

            base.OnAttachedTo(bindable);
        }

        protected override void OnDetachingFrom(ContentPage bindable)
        {
            listView = null;

            base.OnDetachingFrom(bindable);
        }
    }

    public class ChartGridViewBehavior : Behavior<ChartGridView>
    {
        private readonly Audit Audit;
        private SfLinearProgressBar barraProgressoDomande;
        private ChartGridView chartGridView;
        private ChartViewModel chartViewModel;

        private SfChart FirstChart;
        private PieSeries FirstChartSeries;
        private SfChart SecondChart;
        private DoughnutSeries SecondChartSeries;
        private SfChart ThirdChart;
        private BarSeries ThirdChartSeries;
        private SfChart FourthChart;
        private BarSeries FourthChartSeries;

        public ChartGridViewBehavior()
        {
        }

        public ChartGridViewBehavior(Audit item)
        {
            Audit = item;
        }

        protected override void OnAttachedTo(ChartGridView bindable)
        {
            base.OnAttachedTo(bindable);

            chartGridView = bindable;
            chartViewModel = new ChartViewModel(Audit);
            chartGridView.BindingContext = chartViewModel;

            FirstChart = chartGridView.FindByName<SfChart>("FirstChart");

            SecondChart = bindable.FindByName<SfChart>("SecondChart");
            SecondChartSeries = bindable.FindByName<DoughnutSeries>("SecondChartSeries");

            ThirdChart = bindable.FindByName<SfChart>("ThirdChart");

            FourthChart = bindable.FindByName<SfChart>("FourthChart");

            MessagingCenter.Subscribe<ChartViewModel>(this, Messages.OperationFinished.ToString(), (sender) =>
            {
                PopolaGrafici();
            });
        }

        private void PopolaGrafici()
        {
            FirstChartSeries = new PieSeries()
            {
                ItemsSource = chartViewModel.FirstSeriesData,
                XBindingPath = nameof(ChartDataModel.XValue),
                YBindingPath = nameof(ChartDataModel.YValue),
                CircularCoefficient = 0.7,
                ConnectorLineType = ConnectorLineType.Line,
                DataMarkerPosition = CircularSeriesDataMarkerPosition.OutsideExtended,
                EnableAnimation = true,
                EnableSmartLabels = true,
                ExplodeAll = true,
                ListenPropertyChange = true,
                DataMarker = new ChartDataMarker() { LabelContent = LabelContent.YValue, MarkerType = DataMarkerType.Ellipse },
                ColorModel = new ChartColorModel() { CustomBrushes = chartViewModel.FirstColorCollection, Palette = ChartColorPalette.Custom }
            };

            FirstChart.Series.Add(FirstChartSeries);

            SecondChartSeries.ItemsSource = chartViewModel.SecondSeriesData;
            SecondChartSeries.ColorModel = new ChartColorModel() { CustomBrushes = chartViewModel.SecondColorCollection, Palette = ChartColorPalette.Custom };

            ThirdChartSeries = new BarSeries()
            {
                ItemsSource = chartViewModel.ThirdSeriesData,
                ColorModel = new ChartColorModel() { CustomBrushes = chartViewModel.ThirdColorCollection, Palette = ChartColorPalette.Custom },
                EnableAnimation = true,
                IsTransposed = true,
                LegendIcon = ChartLegendIcon.Circle,
                ListenPropertyChange = true,
                Spacing = 0.4,
                XBindingPath = nameof(ChartDataModel.XValue),
                YBindingPath = nameof(ChartDataModel.YValue),
                DataMarker = new ChartDataMarker() { LabelContent = LabelContent.YValue, MarkerType = DataMarkerType.Ellipse }
            };

            ThirdChart.Series.Add(ThirdChartSeries);

            FourthChartSeries = new BarSeries()
            {
                ItemsSource = chartViewModel.FourthSeriesData,
                ColorModel = new ChartColorModel() { CustomBrushes = chartViewModel.FourthColorCollection, Palette = ChartColorPalette.Custom },
                EnableAnimation = true,
                IsTransposed = true,
                LegendIcon = ChartLegendIcon.Circle,
                ListenPropertyChange = true,
                Spacing = 0.4,
                XBindingPath = nameof(ChartDataModel.XValue),
                YBindingPath = nameof(ChartDataModel.YValue),
                DataMarker = new ChartDataMarker() { LabelContent = LabelContent.YValue, MarkerType = DataMarkerType.Ellipse }
            };

            FourthChart.Series.Add(FourthChartSeries);

            barraProgressoDomande = chartGridView.FindByName<SfLinearProgressBar>("BarraProgressoDomande");
            barraProgressoDomande.SetProgress(chartViewModel.ProgressoDomande, 1, Easing.Linear);
        }

        protected override void OnDetachingFrom(ChartGridView bindable)
        {
            chartGridView = null;
            chartViewModel = null;

            base.OnDetachingFrom(bindable);

            MessagingCenter.Unsubscribe<ChartViewModel>(this, Messages.OperationFinished.ToString());
        }
    }
}
