﻿using AuditManager.Helpers;
using Syncfusion.XForms.DataForm;
using System;
using System.ComponentModel;
using Xamarin.Forms;

namespace AuditManager.Behaviors
{
    public class SfDataFormLoginBehavior : Behavior<SfDataForm>
    {
        private SfDataForm dataForm;

        protected override void OnAttachedTo(SfDataForm bindable)
        {
            base.OnAttachedTo(bindable);

            dataForm = bindable;
            dataForm.AutoGeneratingDataFormItem += OnAutoGeneratingDataFormItem;
            dataForm.BindingContextChanged += OnBindingContextChanged;

            dataForm.ValidationMode = ValidationMode.LostFocus;
            dataForm.LayoutManager = new DataFormLayoutManagerExt(dataForm);
        }

        private void OnBindingContextChanged(object sender, EventArgs e)
        {
            if (dataForm.DataObject != null)
                (dataForm.DataObject as INotifyPropertyChanged).PropertyChanged += OnPropertyChanged;
        }

        private void OnAutoGeneratingDataFormItem(object sender, AutoGeneratingDataFormItemEventArgs e)
        {
            if (e.DataFormItem != null)
            {
                if (e.DataFormItem.Name.Equals("HasErrors"))
                    e.Cancel = true;
                else if (e.DataFormItem.Name.Equals("Email"))
                    (e.DataFormItem as DataFormTextItem).KeyBoard = Keyboard.Email;

                if (e.DataFormItem.Name == "Email")
                    e.DataFormItem.ImageSource = FunctionsHelper.SetImageSource("icon_input_mail.png");

                if (e.DataFormItem.Name == "Password")
                    e.DataFormItem.ImageSource = FunctionsHelper.SetImageSource("icon_input_password.png");
                
                if (e.DataFormItem.Name == "DeviceId" || e.DataFormItem.Name == "DeviceName")
                    e.DataFormItem.IsVisible = false;
            }
        }

        private void OnPropertyChanged(object sender, PropertyChangedEventArgs e)
        {

        }

        protected override void OnDetachingFrom(SfDataForm bindable)
        {
            base.OnDetachingFrom(bindable);
            dataForm.AutoGeneratingDataFormItem -= OnAutoGeneratingDataFormItem;
            (dataForm.DataObject as INotifyPropertyChanged).PropertyChanged -= OnPropertyChanged;
            dataForm.BindingContextChanged -= OnBindingContextChanged;
            dataForm = null;
        }
    }

    public class DataFormLayoutManagerExt : DataFormLayoutManager
    {
        public DataFormLayoutManagerExt(SfDataForm dataForm) : base(dataForm)
        {

        }

        protected override View GenerateViewForLabel(DataFormItem dataFormItem)
        {
            var label = base.GenerateViewForLabel(dataFormItem);

            if (label is Label)
                (label as Label).TextColor = Color.White;

            return label;
        }

        protected override void OnEditorCreated(DataFormItem dataFormItem, View editor)
        {
            if (editor is Entry)
            {
                (editor as Entry).TextColor = FunctionsHelper.GetColorFromResource("WhiteColor");
                (editor as Entry).PlaceholderColor = FunctionsHelper.GetColorFromResource("PlaceholderColor");
                (editor as Entry).BackgroundColor = Color.Transparent;
            }
        }
    }
}
