﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Threading.Tasks;

namespace AuditManager.Helpers
{
    public class ReflectionHelper
    {
        public ReflectionHelper()
        {

        }

        public static async Task<List<Type>> GetTypesBy(string assemblyName, string nameSpace)
        {
            var r = new List<Type>();
            Assembly asm = Assembly.Load(new AssemblyName(assemblyName));

            try
            {
                r = asm.GetTypes().Where(w => w != null && w.Namespace != null && w.Namespace.Equals(nameSpace))?.OrderBy(o => o.Name)?.ToList();
            }
            catch (Exception e)
            {
                await FunctionsHelper.Log(nameof(GetTypesBy), e.Message, false);
            }

            return r;
        }

        public static List<string> GetPropertiesNames(Type type)
        {
            var t = GetProperties(type);

            return t.Select(s => s.Name)?.ToList();
        }

        public static List<string> GetPropertiesNames(string type)
        {
            var t = Type.GetType(type);

            return GetPropertiesNames(t);
        }

        public static List<PropertyInfo> GetProperties(Type type)
        {
            return type.GetRuntimeProperties().ToList();

        }

        public static List<PropertyInfo> GetProperties(string type)
        {
            var t = Type.GetType(type);

            return GetProperties(t);
        }

        public static PropertyInfo GetProperty(Type type, string property)
        {
            return GetProperties(type).FirstOrDefault(w => w.Name == property);
        }

        public static string GetPropertyName<T>(Expression<Func<T>> propertyExpression)
        {
            return (propertyExpression.Body as MemberExpression).Member.Name;
        }
    }
}
