using AuditManager.Shared.Types;
using AuditManager.Style;
using Plugin.Settings;
using Plugin.Settings.Abstractions;
using Xamarin.Forms;

namespace AuditManager
{
    public static class Settings
    {
        private static ISettings AppSettings
        {
            get
            {
                return CrossSettings.Current;
            }
        }

        #region Setting Constants
        private const string SettingsKey = "settings_key";
        private const string IsConnectedKey = "connected_key";
        private const string UserKey = "settings_user_key";
        private const string PasswordKey = "settings_password_key";
        private const string IsOnlineKey = "online_key";
        private const string UserNameKey = "username_key";
        private const string UserImageKey = "userimage_key";
        private const string DeviceIdKey = "devicename_key";
        private const string DeviceNameKey = "devicename_key";
        private const string ThemeKey = "theme_key";

        private static readonly string SettingsDefault = string.Empty;
        private static readonly bool IsConnectedDefault = false;
        private static readonly string UserDefault = string.Empty;
        private static readonly string PasswordDefault = string.Empty;
        private static readonly bool IsOnlineDefault = false;
        private static readonly string UserNameDefault = string.Empty;
        private static readonly string UserImageDefault = string.Empty;
        private static readonly string DeviceIdDefault = string.Empty;
        private static readonly string DeviceNameDefault = string.Empty;
        private static readonly int ThemeDefault = 0;
        #endregion

        public static string GeneralSettings
        {
            get { return AppSettings.GetValueOrDefault(SettingsKey, SettingsDefault); }
            set { AppSettings.AddOrUpdateValue(SettingsKey, value); }
        }

        public static string User
        {
            get { return AppSettings.GetValueOrDefault(UserKey, UserDefault); }
            set { AppSettings.AddOrUpdateValue(UserKey, value); }
        }

        public static string Password
        {
            get { return AppSettings.GetValueOrDefault(PasswordKey, PasswordDefault); }
            set { AppSettings.AddOrUpdateValue(PasswordKey, value); }
        }

        public static bool IsConnected
        {
            get { return AppSettings.GetValueOrDefault(IsConnectedKey, IsConnectedDefault); }
            set { AppSettings.AddOrUpdateValue(IsConnectedKey, value); }
        }

        public static bool IsOnline
        {
            get { return AppSettings.GetValueOrDefault(IsOnlineKey, IsOnlineDefault); }
            set { AppSettings.AddOrUpdateValue(IsOnlineKey, value); }
        }

        public static string UserName
        {
            get { return AppSettings.GetValueOrDefault(UserNameKey, UserNameDefault); }
            set { AppSettings.AddOrUpdateValue(UserNameKey, value); }
        }

        public static string UserImage
        {
            get { return AppSettings.GetValueOrDefault(UserImageKey, UserImageDefault); }
            set { AppSettings.AddOrUpdateValue(UserImageKey, value); }
        }

        public static string DeviceId
        {
            get { return AppSettings.GetValueOrDefault(DeviceIdKey, DeviceIdDefault); }
            set { AppSettings.AddOrUpdateValue(DeviceIdKey, value); }
        }

        public static string DeviceName
        {
            get { return AppSettings.GetValueOrDefault(DeviceNameKey, DeviceNameDefault); }
            set { AppSettings.AddOrUpdateValue(DeviceNameKey, value); }
        }

        public static int Theme
        {
            get { return AppSettings.GetValueOrDefault(ThemeKey, ThemeDefault); }
            set { AppSettings.AddOrUpdateValue(ThemeKey, value); }
        }

        public static void SetTheme(Themes theme)
        {
            Device.BeginInvokeOnMainThread(() =>
            {
                Application.Current.Resources.MergedDictionaries.Clear();

                switch (theme)
                {
                    case Themes.Green:
                        Application.Current.Resources.MergedDictionaries.Add(new GreenTheme());
                        break;
                    case Themes.Blue:
                        Application.Current.Resources.MergedDictionaries.Add(new BlueTheme());
                        break;
                    case Themes.Gold:
                        Application.Current.Resources.MergedDictionaries.Add(new GreenTheme());
                        break;
                    case Themes.Red:
                        Application.Current.Resources.MergedDictionaries.Add(new GreenTheme());
                        break;
                    case Themes.White:
                        Application.Current.Resources.MergedDictionaries.Add(new GreenTheme());
                        break;
                    default:
                        Application.Current.Resources.MergedDictionaries.Add(new GreenTheme());
                        break;
                }
            });
        }
    }
}
