﻿using Newtonsoft.Json;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text.RegularExpressions;

namespace AuditManager.Models
{
    [Serializable]
    public class BaseModel : INotifyPropertyChanged, INotifyDataErrorInfo
    {
        [JsonIgnore]
        public Regex emailRegex = new Regex(@"^([\w\.\-]+)@([\w\-]+)((\.(\w){2,3})+)$");
        [JsonIgnore]
        public Dictionary<string, List<string>> propErrors = new Dictionary<string, List<string>>();

        public virtual IEnumerable GetErrors(string propertyName)
        {
            return null;
        }

        [JsonIgnore]
        public bool HasErrors
        {
            get
            {
                var propErrorsCount = propErrors.Values.FirstOrDefault(r => r.Count > 0);
                if (propErrorsCount != null)
                    return true;
                else
                    return false;
            }
        }

        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;
        public event PropertyChangedEventHandler PropertyChanged;

        public void RaisePropertyChanged(string propName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propName));
        }

        public void RaiseErrorChanged(string propName)
        {
            ErrorsChanged?.Invoke(this, new DataErrorsChangedEventArgs(propName));
        }
    }
}
