﻿using AuditManager.Helpers;
using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.Shared.Extensions;
using AuditManager.Views;
using Syncfusion.XForms.TabView;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Xamarin.Forms;
using Xamarin.Forms.Internals;

namespace AuditManager.ViewModels
{
    [Preserve(AllMembers = true)]
    public class AuditListViewModel : BaseViewModel
    {
        #region Properties
        private TabItemCollection listaTabItem;
        public TabItemCollection ListaTabItem
        {
            get { return listaTabItem; }
            set
            {
                listaTabItem = value;
                OnPropertyChanged(nameof(ListaTabItem));
            }
        }

        private ObservableCollection<Stati> listaStati;
        public ObservableCollection<Stati> ListaStati
        {
            get { return listaStati; }
            set
            {
                listaStati = value;
                OnPropertyChanged(nameof(ListaStati));
            }
        }

        private ObservableCollection<Audit> audits;
        public ObservableCollection<Audit> Audits
        {
            get { return audits; }
            set
            {
                audits = value;
                OnPropertyChanged(nameof(Audits));
            }
        }

        private int contaTabItem;
        public int ContaTabItem
        {
            get { return contaTabItem; }
            set
            {
                contaTabItem = value;
                OnPropertyChanged(nameof(ContaTabItem));
            }
        }

        private int tabItemSelezionato;
        public int TabItemSelezionato
        {
            get { return tabItemSelezionato; }
            set
            {
                tabItemSelezionato = value;
                OnPropertyChanged(nameof(TabItemSelezionato));
            }
        }

        private string nomeUtente;
        public string NomeUtente
        {
            get { return nomeUtente; }
            set
            {
                nomeUtente = value;
                OnPropertyChanged(nameof(NomeUtente));
            }
        }

        private string immagineUtente;
        public string ImmagineUtente
        {
            get { return immagineUtente; }
            set
            {
                immagineUtente = value;
                OnPropertyChanged(nameof(ImmagineUtente));
            }
        }

        private bool isOpen = false;
        public bool IsOpen
        {
            get { return isOpen; }
            set
            {
                isOpen = value;
                OnPropertyChanged(nameof(IsOpen));
            }
        }

        private bool isOpenSecondary = false;
        public bool IsOpenSecondary
        {
            get { return isOpenSecondary; }
            set
            {
                bool oldValue = isOpenSecondary;
                isOpenSecondary = value;
                OnPropertyChanged(nameof(IsOpenSecondary));

                if (!value)
                {
                    IsOpenFilters = value;
                    IsOpenFiltersList = value;
                }

                if (oldValue != isOpenSecondary && !isOpenSecondary)
                    MessagingCenter.Send(this, Messages.ClosedFilterMessage.ToString(), "F");
            }
        }

        private Audit selectedItem;
        public Audit SelectedItem
        {
            get { return selectedItem; }
            set
            {
                selectedItem = value;

                if (selectedItem == null)
                    return;

                SelectedItemCommand.Execute(selectedItem);
                SelectedItem = null;
            }
        }
        #endregion

        #region Command Properties
        public Command<object> MyAuditCommand { get; set; }
        public Command<object> LogoutCommand { get; set; }
        public Command OpenDrawerCommand { get; set; }
        public Command<string> OpenFiltersCommand { get; set; }
        public Command ToggleFilterCommand { get; set; }
        public Command<string> ToggleFilterListCommand { get; set; }
        public Command<Audit> SelectedItemCommand { get; set; }
        #endregion

        public AuditListViewModel()
        {
            IsBusy = true;
            IsVisible = true;
            IsOnline = Settings.IsOnline;

            NomeUtente = Settings.UserName;
            ImmagineUtente = Settings.UserImage;

            listaTabItem = new TabItemCollection();
            ListaTabItem = new TabItemCollection();
            audits = new ObservableCollection<Audit>();
            Audits = new ObservableCollection<Audit>();
            listaStati = new ObservableCollection<Stati>();
            ListaStati = new ObservableCollection<Stati>();

            TabItemSelezionato = 0;
            SelectedItem = null;

            MyAuditCommand = new Command<object>((item) => ExecuteMyAuditCommand(item));
            LogoutCommand = new Command<object>((item) => ExecuteLogoutCommand(item));
            OpenDrawerCommand = new Command(() => ExecuteOpenDrawerCommand());
            OpenFiltersCommand = new Command<string>((item) => ExecuteOpenFiltersCommand(item));
            ToggleFilterCommand = new Command(() => ExecuteToggleFilterCommand());
            ToggleFilterListCommand = new Command<string>((s) => ExecuteToggleFilterListCommand(s));
            SelectedItemCommand = new Command<Audit>((item) => ExecuteSelectedItemCommand(item));

            SyncCommand = new Command(async () => await ExecuteSyncCommand());

            LabelFilter = string.Empty;

            Task.Run(async () =>
            {
                await LoadData();
                await GetFiltersData();

                AddVisible = await App.Database.GetFunzioniApp(37, null);
                FilterVisible = await App.Database.GetFunzioniApp(1, null);
                SearchVisible = await App.Database.GetFunzioniApp(38, null);
                CompletaVisible = false;

                if (IsVisiblePhone)
                {
                    HeightPopUp = 80;
                    MoreButtonsVisible = true;
                }
            });

            IsBusy = false;
        }

        private void ShowAlert(AlertModel am)
        {
            AlertButtonOK = am.OkButton;
            AlertTitle = am.Title;
            AlertText = am.Description;

            MessagingCenter.Send(this, Messages.DisplayAlertMessage.ToString(), am);
        }

        #region Data Methods
        private async Task GetStati()
        {
            ListaTabItem = new TabItemCollection();
            ListaStati = new ObservableCollection<Stati>();

            List<Stati> elencoStati = await App.Database.GetStati();
            int stato = 0;

            foreach (var item in elencoStati)
            {
                if (Device.Idiom == TargetIdiom.Tablet || Device.Idiom == TargetIdiom.Desktop)
                {
                    item.Immagine = item.Immagine.ReplaceWithNullControl("disabled", "dark");
                    item.DescStato = item.DescStato.ToTitleCase();
                    item.Audits = new ObservableCollection<Audit>(Audits.Where(w => w.IdStato == item.IdStato).ToList());

                    if (item.Audits.Count > 0)
                        item.InnerListHeight = 235;
                    else
                        item.InnerListHeight = 0;

                    ListaStati.Add(item);
                }
                else
                {
                    ListaStati.Add(item);
                }

                stato++;
            }

            ContaTabItem = ListaTabItem.Count;
        }

        private async Task GetAudits()
        {
            Audits.Clear();

            List<Audit> listaAudit = await App.Database.GetAuditFilteredAsync(ListaTipiAuditSelezionati.ToList(), ListaSoggettiSelezionati.ToList(), ListaMotivazioniSelezionati.ToList());

            foreach (var item in listaAudit)
            {
                if (Device.Idiom == TargetIdiom.Tablet || Device.Idiom == TargetIdiom.Desktop)
                {
                    item.BloccoApp1 = item.BloccoApp1.ReplaceNewlines(" - ");
                    item.BloccoApp3 = item.BloccoApp3.ReplaceNewlines(" - ");
                }

                Audits.Add(item);
            }
        }

        private async Task GetFiltersData()
        {
            ListaTipiAudit = new ObservableCollection<TipiAudit>(await App.Database.GetTipiAudit());
            ListaSoggetti = new ObservableCollection<Soggetti>(await App.Database.GetSoggettiFilterAsync());
            ListaMotivazioni = new ObservableCollection<Motivazioni>(await App.Database.GetMotivazioni());
        }

        public async Task LoadData(bool reLoad = true)
        {
            await GetAudits();

            if (reLoad || (Device.Idiom == TargetIdiom.Tablet || Device.Idiom == TargetIdiom.Desktop))
                await GetStati();
        }
        #endregion

        #region Command
        private async void ExecuteMyAuditCommand(object item)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteMyAuditCommand), item.ToString());
            MessagingCenter.Send(this, Messages.HomePage.ToString());
            IsOpen = false;
        }

        public async Task ExecuteSyncCommand()
        {
            IsBusy = true;

            await FunctionsHelper.Log(nameof(this.ExecuteSyncCommand), string.Empty);

            await SyncDB();
            await GetFiltersData();
            await LoadData();

            IsBusy = false;
        }

        private async void ExecuteLogoutCommand(object item)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteLogoutCommand), item.ToString());
            //MessagingCenter.Send(this, Messages.DisplayConfirmMessage.ToString(), ConfermaLogout);
            IsOpen = false;
            IsOpenAlert = true;
        }

        private void ExecuteOpenDrawerCommand()
        {
            IsOpen = !IsOpen;
        }

        private void ExecuteToggleFilterCommand()
        {
            IsOpenSecondary = !IsOpenSecondary;
            IsOpenFilters = IsOpenSecondary;
        }

        private void ExecuteToggleFilterListCommand(object s)
        {
            if (s.Equals("R"))
            {
                if (IsVisibleListTipiAudit)
                {
                    foreach (var item in ListaTipiAudit.Where(w => w.IsSelected == true).ToList())
                        item.IsSelected = false;
                }

                if (IsVisibleListSoggetti)
                {
                    foreach (var item in ListaSoggetti.Where(w => w.IsSelected == true).ToList())
                        item.IsSelected = false;
                }

                if (IsVisibleListMotivazioni)
                {
                    foreach (var item in ListaMotivazioni.Where(w => w.IsSelected == true).ToList())
                        item.IsSelected = false;
                }
            }

            ListaTipiAuditSelezionati = new ObservableCollection<TipiAudit>(ListaTipiAudit.Where(w => w.IsSelected == true));
            ListaSoggettiSelezionati = new ObservableCollection<Soggetti>(ListaSoggetti.Where(w => w.IsSelected == true));
            ListaMotivazioniSelezionati = new ObservableCollection<Motivazioni>(ListaMotivazioni.Where(w => w.IsSelected == true));

            if (s.Equals("F"))
            {
                IsOpenSecondary = !IsOpenSecondary;
                IsOpenFilters = false;
            }

            IsOpenFiltersList = false;
            FilterHeight = ConstFilterHeight;
        }

        private void ExecuteOpenFiltersCommand(string item)
        {
            IsOpenFiltersList = true;
            FilterHeight = 350;

            IsVisibleListTipiAudit = item.Equals(Resx.AppResources.LabelFilterTipoAudit);
            IsVisibleListSoggetti = item.Equals(Resx.AppResources.LabelFilterSoggetto);
            IsVisibleListMotivazioni = item.Equals(Resx.AppResources.LabelFilterMotivo);
            IsVisibleListaValoriRisposte = false;

            LabelFilter = item.ToTitleCase();
        }

        private async void ExecuteSelectedItemCommand(Audit item)
        {
            IsBusy = true;

            await Task.Delay(200);

            if (item != null)
                await App.NavigationService.NavigateAsync(nameof(CategoriePage), item);
        }
        #endregion
    }
}