﻿using AuditManager.Helpers;
using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.Services;
using AuditManager.Shared.Types;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace AuditManager.ViewModels
{
    public class BaseViewModel : INotifyPropertyChanged
    {
        #region Props
        public const double ConstFilterHeight = 250;

        private bool isVisible;
        public bool IsVisible
        {
            get { return isVisible; }
            set
            {
                isVisible = value;
                OnPropertyChanged(nameof(IsVisible));
            }
        }

        private bool isEmpty;
        public bool IsEmpty
        {
            get { return isEmpty; }
            set
            {
                isEmpty = value;
                OnPropertyChanged(nameof(IsEmpty));
            }
        }

        private bool _isBusy;
        public bool IsBusy
        {
            get { return _isBusy; }
            set
            {
                _isBusy = value;
                OnPropertyChanged(nameof(IsBusy));
            }
        }

        private bool isEnabled;
        public bool IsEnabled
        {
            get { return isEnabled; }
            set
            {
                isEnabled = value;
                OnPropertyChanged(nameof(IsEnabled));
            }
        }

        private bool isOnline;
        public bool IsOnline
        {
            get { return isOnline; }
            set
            {
                isOnline = value;
                OnPropertyChanged(nameof(IsOnline));

                if (value)
                {
                    LabelDataOnlineOffline = Resx.AppResources.StringOnlineData;
                    LabelOnlineOffline = Resx.AppResources.StringOffline;
                    ImageOnlineOffline = (FileImageSource)FunctionsHelper.SetImageSource("icon_offline_small_light.png");
                }
                else
                {
                    LabelDataOnlineOffline = Resx.AppResources.StringOfflineData;
                    LabelOnlineOffline = Resx.AppResources.StringOnline;
                    ImageOnlineOffline = (FileImageSource)FunctionsHelper.SetImageSource("icon_online_big_light.png");
                }
            }
        }

        private bool isVisiblePhone;
        public bool IsVisiblePhone
        {
            get { return isVisiblePhone; }
            set
            {
                isVisiblePhone = value;
                OnPropertyChanged(nameof(IsVisiblePhone));
            }
        }

        public NetworkService ns = new NetworkService();
        #endregion

        #region TopHeaderBar Properties
        private bool searchVisible = false;
        public bool SearchVisible
        {
            get { return searchVisible; }
            set
            {
                searchVisible = value;
                OnPropertyChanged(nameof(SearchVisible));
            }
        }

        private bool filterVisible = false;
        public bool FilterVisible
        {
            get { return filterVisible; }
            set
            {
                filterVisible = value;
                OnPropertyChanged(nameof(FilterVisible));
            }
        }

        private bool addVisible = false;
        public bool AddVisible
        {
            get { return addVisible; }
            set
            {
                addVisible = value;
                OnPropertyChanged(nameof(AddVisible));
            }
        }

        private bool dettaglioAuditVisible = false;
        public bool DettaglioAuditVisible
        {
            get { return dettaglioAuditVisible; }
            set
            {
                dettaglioAuditVisible = value;
                OnPropertyChanged(nameof(DettaglioAuditVisible));
            }
        }

        private bool allegatoPrincipaleVisible = false;
        public bool AllegatoPrincipaleVisible
        {
            get { return allegatoPrincipaleVisible; }
            set
            {
                allegatoPrincipaleVisible = value;
                OnPropertyChanged(nameof(AllegatoPrincipaleVisible));
            }
        }

        private string labelDataOnlineOffline;
        public string LabelDataOnlineOffline
        {
            get { return labelDataOnlineOffline; }
            set
            {
                labelDataOnlineOffline = value;
                OnPropertyChanged(nameof(LabelDataOnlineOffline));
            }
        }

        private string labelOnlineOffline;
        public string LabelOnlineOffline
        {
            get { return labelOnlineOffline; }
            set
            {
                labelOnlineOffline = value;
                OnPropertyChanged(nameof(LabelOnlineOffline));
            }
        }

        private string imageOnlineOffline;
        public string ImageOnlineOffline
        {
            get { return imageOnlineOffline; }
            set
            {
                imageOnlineOffline = value;
                OnPropertyChanged(nameof(ImageOnlineOffline));
            }
        }

        private bool completaVisible = true;
        public bool CompletaVisible
        {
            get { return completaVisible; }
            set
            {
                completaVisible = value;
                OnPropertyChanged(nameof(CompletaVisible));
            }
        }

        private bool stampaVisible = false;
        public bool StampaVisible
        {
            get { return stampaVisible; }
            set
            {
                stampaVisible = value;
                OnPropertyChanged(nameof(StampaVisible));
            }
        }

        private bool listaLavoriVisible = false;
        public bool ListaLavoriVisible
        {
            get { return listaLavoriVisible; }
            set
            {
                listaLavoriVisible = value;
                OnPropertyChanged(nameof(ListaLavoriVisible));
            }
        }

        private bool notificaListaLavoriVisible = false;
        public bool NotificaListaLavoriVisible
        {
            get { return notificaListaLavoriVisible; }
            set
            {
                notificaListaLavoriVisible = value;
                OnPropertyChanged(nameof(NotificaListaLavoriVisible));
            }
        }

        private bool convalidaVisible = false;
        public bool ConvalidaVisible
        {
            get { return convalidaVisible; }
            set
            {
                convalidaVisible = value;
                OnPropertyChanged(nameof(ConvalidaVisible));
            }
        }

        private bool addDomandaVisible = false;
        public bool AddDomandaVisible
        {
            get { return addDomandaVisible; }
            set
            {
                addDomandaVisible = value;
                OnPropertyChanged(nameof(AddDomandaVisible));
            }
        }

        private bool rispostaMultiplaVisible = false;
        public bool RispostaMultiplaVisible
        {
            get { return rispostaMultiplaVisible; }
            set
            {
                rispostaMultiplaVisible = value;
                OnPropertyChanged(nameof(RispostaMultiplaVisible));
            }
        }

        private double heightPopUp;
        public double HeightPopUp
        {
            get { return heightPopUp; }
            set
            {
                heightPopUp = value;
                OnPropertyChanged(nameof(HeightPopUp));
            }
        }

        private bool moreButtonsVisible = false;
        public bool MoreButtonsVisible
        {
            get { return moreButtonsVisible; }
            set
            {
                moreButtonsVisible = value;
                OnPropertyChanged(nameof(MoreButtonsVisible));
            }
        }
        #endregion

        #region Alert Properties
        private bool isOpenAlert;
        public bool IsOpenAlert
        {
            get { return isOpenAlert; }
            set
            {
                isOpenAlert = value;
                OnPropertyChanged(nameof(IsOpenAlert));
            }
        }

        private string alertTitle;
        public string AlertTitle
        {
            get { return alertTitle; }
            set
            {
                alertTitle = value;
                OnPropertyChanged(nameof(AlertTitle));
            }
        }

        private string alertText;
        public string AlertText
        {
            get { return alertText; }
            set
            {
                alertText = value;
                OnPropertyChanged(nameof(AlertText));
            }
        }

        private string alertButtonOK;
        public string AlertButtonOK
        {
            get { return alertButtonOK; }
            set
            {
                alertButtonOK = value;
                OnPropertyChanged(nameof(AlertButtonOK));
            }
        }

        private string alertButtonKO;
        public string AlertButtonKO
        {
            get { return alertButtonKO; }
            set
            {
                alertButtonKO = value;
                OnPropertyChanged(nameof(AlertButtonKO));
            }
        }

        public Command AcceptButtonCustomCommand { get; set; }
        public Command DeclineButtonCustomCommand { get; set; }
        #endregion

        #region Filters Properties
        private bool isOpenFilters = false;
        public bool IsOpenFilters
        {
            get { return isOpenFilters; }
            set
            {
                isOpenFilters = value;
                OnPropertyChanged(nameof(IsOpenFilters));

                if (!value)
                {
                    IsOpenFiltersList = value;
                    FilterHeight = ConstFilterHeight;
                }
            }
        }

        private bool isOpenFiltersList = false;
        public bool IsOpenFiltersList
        {
            get { return isOpenFiltersList; }
            set
            {
                isOpenFiltersList = value;
                OnPropertyChanged(nameof(IsOpenFiltersList));
            }
        }

        private ObservableCollection<TipiAudit> listaTipiAudit;
        public ObservableCollection<TipiAudit> ListaTipiAudit
        {
            get { return listaTipiAudit; }
            set
            {
                listaTipiAudit = value;
                OnPropertyChanged(nameof(ListaTipiAudit));
            }
        }

        private ObservableCollection<Soggetti> listaSoggetti;
        public ObservableCollection<Soggetti> ListaSoggetti
        {
            get { return listaSoggetti; }
            set
            {
                listaSoggetti = value;
                OnPropertyChanged(nameof(ListaSoggetti));
            }
        }

        private ObservableCollection<Motivazioni> listaMotivazioni;
        public ObservableCollection<Motivazioni> ListaMotivazioni
        {
            get { return listaMotivazioni; }
            set
            {
                listaMotivazioni = value;
                OnPropertyChanged(nameof(ListaMotivazioni));
            }
        }

        private ObservableCollection<ValoriRisposte> listaValoriRisposte;
        public ObservableCollection<ValoriRisposte> ListaValoriRisposte
        {
            get { return listaValoriRisposte; }
            set
            {
                listaValoriRisposte = value;
                OnPropertyChanged(nameof(ListaValoriRisposte));
            }
        }

        private bool isVisibleListTipiAudit = false;
        public bool IsVisibleListTipiAudit
        {
            get { return isVisibleListTipiAudit; }
            set
            {
                isVisibleListTipiAudit = value;
                OnPropertyChanged(nameof(IsVisibleListTipiAudit));
            }
        }

        private bool isVisibleListSoggetti = false;
        public bool IsVisibleListSoggetti
        {
            get { return isVisibleListSoggetti; }
            set
            {
                isVisibleListSoggetti = value;
                OnPropertyChanged(nameof(IsVisibleListSoggetti));
            }
        }

        private bool isVisibleListMotivazioni = false;
        public bool IsVisibleListMotivazioni
        {
            get { return isVisibleListMotivazioni; }
            set
            {
                isVisibleListMotivazioni = value;
                OnPropertyChanged(nameof(IsVisibleListMotivazioni));
            }
        }

        private bool isVisibleListaValoriRisposte = false;
        public bool IsVisibleListaValoriRisposte
        {
            get { return isVisibleListaValoriRisposte; }
            set
            {
                isVisibleListaValoriRisposte = value;
                OnPropertyChanged(nameof(IsVisibleListaValoriRisposte));
            }
        }

        private string labelFilter;
        public string LabelFilter
        {
            get { return labelFilter; }
            set
            {
                labelFilter = value;
                OnPropertyChanged(nameof(LabelFilter));
            }
        }

        private ObservableCollection<TipiAudit> listaTipiAuditSelezionati;
        public ObservableCollection<TipiAudit> ListaTipiAuditSelezionati
        {
            get { return listaTipiAuditSelezionati; }
            set
            {
                listaTipiAuditSelezionati = value;
                OnPropertyChanged(nameof(ListaTipiAuditSelezionati));
            }
        }

        private ObservableCollection<Soggetti> listaSoggettiSelezionati;
        public ObservableCollection<Soggetti> ListaSoggettiSelezionati
        {
            get { return listaSoggettiSelezionati; }
            set
            {
                listaSoggettiSelezionati = value;
                OnPropertyChanged(nameof(ListaSoggettiSelezionati));
            }
        }

        private ObservableCollection<Motivazioni> listaMotivazioniSelezionati;
        public ObservableCollection<Motivazioni> ListaMotivazioniSelezionati
        {
            get { return listaMotivazioniSelezionati; }
            set
            {
                listaMotivazioniSelezionati = value;
                OnPropertyChanged(nameof(ListaMotivazioniSelezionati));
            }
        }

        private ObservableCollection<ValoriRisposte> listaValoriRisposteSelezionati;
        public ObservableCollection<ValoriRisposte> ListaValoriRisposteSelezionati
        {
            get { return listaValoriRisposteSelezionati; }
            set
            {
                listaValoriRisposteSelezionati = value;
                OnPropertyChanged(nameof(ListaValoriRisposteSelezionati));
            }
        }

        private double filterHeight;
        public double FilterHeight
        {
            get { return filterHeight; }
            set
            {
                filterHeight = value;
                OnPropertyChanged(nameof(FilterHeight));
            }
        }
        #endregion

        public BaseViewModel()
        {
            IsVisible = true;
            IsEmpty = false;
            IsBusy = false;
            IsOpenAlert = false;

            MessagingCenter.Subscribe<App>(this, Messages.ConnectionMessage.ToString(), (sender) =>
            {
                IsEnabled = Settings.IsConnected;
                IsOnline = Settings.IsConnected;

                Settings.IsOnline = (Settings.IsOnline && Settings.IsConnected);
            });

            AcceptButtonCustomCommand = new Command(() => ExecuteAcceptCommand());
            DeclineButtonCustomCommand = new Command(() => ExecuteDeclineCommand());
            OfflineCommand = new Command(() => ExecuteOfflineCommand());

            if (Device.Idiom == TargetIdiom.Tablet || Device.Idiom == TargetIdiom.Desktop)
                IsVisiblePhone = false;
            else
                IsVisiblePhone = true;

            HeightPopUp = 0;
            MoreButtonsVisible = false;

            listaTipiAudit = new ObservableCollection<TipiAudit>();
            ListaTipiAudit = new ObservableCollection<TipiAudit>();
            listaSoggetti = new ObservableCollection<Soggetti>();
            ListaSoggetti = new ObservableCollection<Soggetti>();
            listaMotivazioni = new ObservableCollection<Motivazioni>();
            ListaMotivazioni = new ObservableCollection<Motivazioni>();
            listaTipiAuditSelezionati = new ObservableCollection<TipiAudit>();
            ListaTipiAuditSelezionati = new ObservableCollection<TipiAudit>();
            listaSoggettiSelezionati = new ObservableCollection<Soggetti>();
            ListaSoggettiSelezionati = new ObservableCollection<Soggetti>();
            listaMotivazioniSelezionati = new ObservableCollection<Motivazioni>();
            ListaMotivazioniSelezionati = new ObservableCollection<Motivazioni>();

            listaValoriRisposte = new ObservableCollection<ValoriRisposte>();
            ListaValoriRisposte = new ObservableCollection<ValoriRisposte>()
            {
                new ValoriRisposte { IdValoreRisposta = 0, DescValoreRisposta = Resx.AppResources.LabelStatoKO, IsSelected = false },
                new ValoriRisposte { IdValoreRisposta = 1, DescValoreRisposta = Resx.AppResources.LabelStatoOK, IsSelected = false },
                new ValoriRisposte { IdValoreRisposta = 2, DescValoreRisposta = Resx.AppResources.LabelStatoNA, IsSelected = false },
                new ValoriRisposte { IdValoreRisposta = 3, DescValoreRisposta = Resx.AppResources.LabelStatoNT, IsSelected = false },
                new ValoriRisposte { IdValoreRisposta = 4, DescValoreRisposta = Resx.AppResources.LabelStatoDV, IsSelected = false }
            };
            listaValoriRisposteSelezionati = new ObservableCollection<ValoriRisposte>();
            ListaValoriRisposteSelezionati = new ObservableCollection<ValoriRisposte>();

            FilterHeight = ConstFilterHeight;
        }

        #region Common Methods
        public async Task<ApiResponse<bool>> SyncDB(bool syncDb = false)
        {
            ApiResponse<bool> syncDatabase = await ns.GetDatabaseFromServer(syncDb);
            return syncDatabase;
        }
        #endregion

        #region Common Commands
        public Command OfflineCommand { get; set; }
        public Command SyncCommand { get; set; }
        public Command DettaglioAuditCommand { get; set; }
        public Command AllegatoPrincipaleCommand { get; set; }
        public Command CompletaCommand { get; set; }
        public Command ListaLavoriCommand { get; set; }
        public Command NotificaListaLavoriCommand { get; set; }
        public Command ConvalidaCommand { get; set; }
        public Command StampaCommand { get; set; }

        public async void ExecuteOfflineCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteOfflineCommand), string.Empty);
            ToggleOffline();
        }

        public void ToggleOffline()
        {
            Settings.IsOnline = !Settings.IsOnline;
            IsOnline = Settings.IsOnline;
        }
        #endregion

        #region Alert Commands
        private void ExecuteAcceptCommand()
        {
            FunctionsHelper.Logout();
        }

        private void ExecuteDeclineCommand()
        {
            IsOpenAlert = false;
        }
        #endregion

        #region PropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion
    }

    public class ValoriRisposte : INotifyPropertyChanged
    {
        private int idValoreRisposta;
        public int IdValoreRisposta
        {
            get { return idValoreRisposta; }
            set
            {
                idValoreRisposta = value;
                OnPropertyChanged(nameof(IdValoreRisposta));
            }
        }

        private string descValoreRisposta;
        public string DescValoreRisposta
        {
            get { return descValoreRisposta; }
            set
            {
                descValoreRisposta = value;
                OnPropertyChanged(nameof(DescValoreRisposta));
            }
        }

        private bool? isSelected;
        public bool? IsSelected
        {
            get { return isSelected; }
            set
            {
                isSelected = value;
                OnPropertyChanged(nameof(IsSelected));
            }
        }


        public ValoriRisposte()
        {

        }

        #region PropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion
    }
}