﻿using AuditManager.Helpers;
using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.Shared.Extensions;
using AuditManager.Views;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace AuditManager.ViewModels
{
    public class CategorieViewModel : BaseViewModel
    {
        #region Properties
        private Audit audit;
        public Audit Audit
        {
            get { return audit; }
            set
            {
                audit = value;
                OnPropertyChanged(nameof(Audit));
            }
        }

        private ObservableCollection<AuditCategorie> listaCategorie;
        public ObservableCollection<AuditCategorie> ListaCategorie
        {
            get { return listaCategorie; }
            set
            {
                listaCategorie = value;
                OnPropertyChanged(nameof(ListaCategorie));
            }
        }

        private string immaginePagina;
        public string ImmaginePagina
        {
            get { return immaginePagina; }
            set
            {
                immaginePagina = value;
                OnPropertyChanged(nameof(ImmaginePagina));
            }
        }

        private bool isVisibleListaCategorie;
        public bool IsVisibleListaCategorie
        {
            get { return isVisibleListaCategorie; }
            set
            {
                isVisibleListaCategorie = value;
                OnPropertyChanged(nameof(IsVisibleListaCategorie));
            }
        }

        private bool isOpen = false;
        public bool IsOpen
        {
            get { return isOpen; }
            set
            {
                bool oldValue = isOpen;
                isOpen = value;
                OnPropertyChanged(nameof(IsOpen));

                if (oldValue != isOpen && !isOpen)
                    MessagingCenter.Send(this, Messages.ClosedFilterMessage.ToString(), "F");
            }
        }
        #endregion

        #region Commands
        public Command BackCommand { get; set; }
        public Command SwitchCommand { get; set; }

        public Command<string> OpenFiltersCommand { get; set; }
        public Command ToggleFilterCommand { get; set; }
        public Command<string> ToggleFilterListCommand { get; set; }

        public Command<object> NavigateCommand { get; set; }
        #endregion


        public CategorieViewModel() : base()
        {

        }

        public CategorieViewModel(Audit audit) : base()
        {
            IsVisible = true;
            IsBusy = false;
            IsOnline = Settings.IsOnline;

            AddVisible = false;
            SearchVisible = false;

            Audit = audit;

            Audit.BloccoApp1 = Audit.BloccoApp1.ReplaceNewlines(" - ");
            Audit.BloccoApp4 = $"{Audit.BloccoApp2} - {Audit.BloccoApp3}";
            Audit.BloccoApp4 = Audit.BloccoApp4.ReplaceNewlines(" - ");

            IsVisibleListaCategorie = true;
            ImmaginePagina = FunctionsHelper.SetImageString("icon_indicatori_big_light.png");

            listaCategorie = new ObservableCollection<AuditCategorie>();
            ListaCategorie = new ObservableCollection<AuditCategorie>();

            DettaglioAuditCommand = new Command(async () => await ExecuteDettaglioAuditCommand());
            AllegatoPrincipaleCommand = new Command(async () => await ExecuteAllegatoPrincipaleCommand());
            CompletaCommand = new Command(async () => await ExecuteCompletaCommand());
            StampaCommand = new Command(async () => await ExecuteStampaCommand());
            ListaLavoriCommand = new Command(async () => await ExecuteListaLavoriCommand());
            NotificaListaLavoriCommand = new Command(async () => await ExecuteNotificaListaLavoriCommand());
            ConvalidaCommand = new Command(async () => await ExecuteConvalidaCommand());
            SyncCommand = new Command(async () => await ExecuteSyncCommand());

            ToggleFilterCommand = new Command(() => ExecuteToggleFilterCommand());
            OpenFiltersCommand = new Command<string>((item) => ExecuteOpenFiltersCommand(item));
            ToggleFilterListCommand = new Command<string>((s) => ExecuteToggleFilterListCommand(s));

            NavigateCommand = new Command<object>(async (s) => await ExecuteNavigateCommand(s));

            BackCommand = new Command(async () => await ExecuteBackCommand());
            SwitchCommand = new Command(() => ExecuteSwitchCommand());

            Task.Run(async () =>
            {
                await GetCategorie();

                FilterVisible = true;
                AllegatoPrincipaleVisible = true;
                RispostaMultiplaVisible = false;

                List<AuditTipiAudit> listaTipiAudit = await App.Database.GetAuditTipiAudit(Audit.IdAudit);

                foreach (var item in listaTipiAudit)
                {
                    if (!DettaglioAuditVisible)
                        DettaglioAuditVisible = await App.Database.GetFunzioniApp(3, item.IdTipoAudit);

                    if (!StampaVisible)
                        StampaVisible = await App.Database.GetFunzioniApp(4, item.IdTipoAudit);

                    if (!ListaLavoriVisible)
                        ListaLavoriVisible = await App.Database.GetFunzioniApp(5, item.IdTipoAudit);

                    if (!NotificaListaLavoriVisible)
                        NotificaListaLavoriVisible = await App.Database.GetFunzioniApp(6, item.IdTipoAudit);

                    if (!ConvalidaVisible)
                        ConvalidaVisible = await App.Database.GetFunzioniApp(39, item.IdTipoAudit);
                }

                if (IsVisiblePhone)
                {
                    MoreButtonsVisible = true;
                    HeightPopUp = 80;
                }

                if (IsVisiblePhone && DettaglioAuditVisible)
                    HeightPopUp += 40;

                if (IsVisiblePhone && AllegatoPrincipaleVisible)
                    HeightPopUp += 40;

                if (CompletaVisible)
                    HeightPopUp += 40;

                if (StampaVisible)
                    HeightPopUp += 40;

                if (ListaLavoriVisible)
                    HeightPopUp += 40;

                if (NotificaListaLavoriVisible)
                    HeightPopUp += 40;

                if (ConvalidaVisible)
                    HeightPopUp += 40;

                await GetConteggioCategorie();

                if (Audit.IdStato == 1)
                    await FunctionsHelper.UpdateStatoAudit(Audit, 2);
            });
        }

        public async Task GetCategorie()
        {
            List<AuditCategorie> lista = await App.Database.GetAuditCategorie(Audit.IdAudit);

            List<AuditCategorie> listaFinale = new List<AuditCategorie>();

            if (ListaValoriRisposteSelezionati.Count == 0)
                listaFinale.AddRange(lista);
            else
            {
                foreach (var selezionato in ListaValoriRisposteSelezionati)
                {
                    switch (selezionato.IdValoreRisposta)
                    {
                        case 0:
                            listaFinale.AddRange(lista.Where(w => w.NumeroKO > 0).ToList());
                            break;
                        case 1:
                            listaFinale.AddRange(lista.Where(w => w.NumeroOK > 0).ToList());
                            break;
                        case 2:
                            listaFinale.AddRange(lista.Where(w => w.NumeroNA > 0).ToList());
                            break;
                        case 3:
                            listaFinale.AddRange(lista.Where(w => w.NumeroNT > 0).ToList());
                            break;
                        case 4:
                            listaFinale.AddRange(lista.Where(w => w.NumeroDV > 0).ToList());
                            break;
                        default:
                            break;
                    }
                }
            }

            ListaCategorie = new ObservableCollection<AuditCategorie>(listaFinale);
        }

        public async Task GetConteggioCategorie()
        {
            foreach (var item in ListaCategorie)
            {
                item.NumeroOK = await App.Database.GetCountersAudit(item.IdAudit, item.IdCategoria, "1");
                item.NumeroKO = await App.Database.GetCountersAudit(item.IdAudit, item.IdCategoria, "0");
                item.NumeroNA = await App.Database.GetCountersAudit(item.IdAudit, item.IdCategoria, "2");
                item.NumeroNT = await App.Database.GetCountersAudit(item.IdAudit, item.IdCategoria, "3");
                item.NumeroDV = await App.Database.GetCountersAudit(item.IdAudit, item.IdCategoria, "");

                if (item.NumeroKO > 0)
                    item.ColoreCategoria = "ColoreStatoKO";
                else
                {
                    if (item.NumeroNT > 0)
                        item.ColoreCategoria = "ColoreStatoNT";
                    else
                    {
                        if (item.NumeroOK > 0)
                            item.ColoreCategoria = "ColoreStatoOK";
                        else
                        {
                            if (item.NumeroNA > 0)
                                item.ColoreCategoria = "ColoreStatoNA";
                            else
                                item.ColoreCategoria = "ColoreStatoDV";
                        }
                    }
                }

                if (item.NumeroKO > 0)
                    item.BadgeKOVisibile = true;

                if (item.NumeroNT > 0)
                    item.BadgeNTVisibile = true;

                if (item.NumeroOK > 0)
                    item.BadgeOKVisibile = true;

                if (item.NumeroNA > 0)
                    item.BadgeNAVisibile = true;

                if (item.NumeroDV > 0)
                    item.BadgeDVVisibile = true;
            }
        }

        #region Command
        private void ExecuteToggleFilterCommand()
        {
            IsOpen = !IsOpen;
            IsOpenFilters = IsOpen;
        }

        private void ExecuteOpenFiltersCommand(string item)
        {
            IsOpenFiltersList = true;
            FilterHeight = 350;

            IsVisibleListTipiAudit = false;
            IsVisibleListSoggetti = false;
            IsVisibleListMotivazioni = false;
            IsVisibleListaValoriRisposte = true;

            LabelFilter = item.ToTitleCase();
        }

        private void ExecuteToggleFilterListCommand(object s)
        {
            if (s.Equals("R"))
            {
                foreach (var item in ListaValoriRisposte.Where(w => w.IsSelected == true).ToList())
                    item.IsSelected = false;
            }

            ListaValoriRisposteSelezionati = new ObservableCollection<ValoriRisposte>(ListaValoriRisposte.Where(w => w.IsSelected == true));

            if (s.Equals("F"))
            {
                IsOpen = !IsOpen;
                IsOpenFilters = false;
            }

            IsOpenFiltersList = false;
            FilterHeight = ConstFilterHeight;
        }

        private async Task ExecuteBackCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteBackCommand), string.Empty);

            await App.NavigationService.NavigateAsync(nameof(AuditListPage));
        }

        private void ExecuteSwitchCommand()
        {
            if (IsVisibleListaCategorie)
                ImmaginePagina = FunctionsHelper.SetImageString("icon_categorie_big_light.png");
            else
                ImmaginePagina = FunctionsHelper.SetImageString("icon_indicatori_big_light.png");

            IsVisibleListaCategorie = !IsVisibleListaCategorie;
        }

        private async Task ExecuteSyncCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteSyncCommand), string.Empty);

            await SyncDB();
            await GetCategorie();
            await GetConteggioCategorie();
        }

        private async Task ExecuteAllegatoPrincipaleCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteAllegatoPrincipaleCommand), string.Empty);
        }

        private async Task ExecuteDettaglioAuditCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteDettaglioAuditCommand), string.Empty);

            //await App.NavigationService.NavigateAsync("DettaglioAuditPage", Audit);
        }

        private async Task ExecuteStampaCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteStampaCommand), string.Empty);

            //await App.NavigationService.NavigateAsync("StampaAuditPage", Audit);
        }

        private async Task ExecuteConvalidaCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteConvalidaCommand), string.Empty);

            //await App.NavigationService.NavigateAsync("ConvalidaAuditPage", Audit);
        }

        private async Task ExecuteListaLavoriCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteListaLavoriCommand), string.Empty);

            //await App.NavigationService.NavigateAsync("ListaLavoriPage", Audit);
        }

        private async Task ExecuteNotificaListaLavoriCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteNotificaListaLavoriCommand), string.Empty);
        }

        private async Task ExecuteCompletaCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteCompletaCommand), string.Empty);

            //await App.NavigationService.NavigateAsync("CompletaAuditPage", Audit);
        }

        private async Task ExecuteNavigateCommand(object obj)
        {
            if (obj != null)
            {
                Syncfusion.ListView.XForms.ItemTappedEventArgs item = obj as Syncfusion.ListView.XForms.ItemTappedEventArgs;
                AuditCategorie categoria = item.ItemData as AuditCategorie;
                NavigationParameterModel parameterModel = new NavigationParameterModel
                {
                    Audit = Audit,
                    Categoria = categoria
                };

                await App.NavigationService.NavigateAsync(nameof(QuestionarioPage), parameterModel);
            }
        }
        #endregion
    }
}