﻿using AuditManager.Helpers;
using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.Shared.Constants;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace AuditManager.ViewModels
{
    public class ChartViewModel : BaseViewModel
    {
        public Audit Audit { get; set; }

        public double ProgressoDomande { get; set; }

        private bool _IsViewModelLoaded;
        public bool IsViewModelLoaded
        {
            get { return _IsViewModelLoaded; }
            set
            {
                _IsViewModelLoaded = value;
                OnPropertyChanged(nameof(IsViewModelLoaded));
            }
        }

        private bool _IsThirdChartVisible = false;
        public bool IsThirdChartVisible
        {
            get { return _IsThirdChartVisible; }
            set
            {
                _IsThirdChartVisible = value;
                OnPropertyChanged(nameof(IsThirdChartVisible));
            }
        }

        private bool _IsFourthChartVisible = false;
        public bool IsFourthChartVisible
        {
            get { return _IsFourthChartVisible; }
            set
            {
                _IsFourthChartVisible = value;
                OnPropertyChanged(nameof(IsFourthChartVisible));
            }
        }

        public ObservableCollection<ChartDataModel> FirstSeriesData { get; set; }
        public ObservableCollection<ChartDataModel> SecondSeriesData { get; set; }
        public ObservableCollection<ChartDataModel> ThirdSeriesData { get; set; }
        public ObservableCollection<ChartDataModel> FourthSeriesData { get; set; }

        public ObservableCollection<Color> FirstColorCollection { get; set; }
        public ObservableCollection<Color> SecondColorCollection { get; set; }
        public ObservableCollection<Color> ThirdColorCollection { get; set; }
        public ObservableCollection<Color> FourthColorCollection { get; set; }

        public ChartViewModel()
        {

        }

        public ChartViewModel(Audit audit)
        {
            IsViewModelLoaded = false;
            Audit = audit;
            ProgressoDomande = 0;

            Task.Run(async () =>
            {
                await GetChartVisibility();
                await GetChartData();

                if (IsViewModelLoaded)
                    MessagingCenter.Send(this, Messages.OperationFinished.ToString());
            });
        }

        private async Task GetChartVisibility()
        {
            ParametriGlobali pgTerzoGrafico = await App.Database.GetParametroGlobale(Constants.ConstTerzoGrafico);
            ParametriGlobali pgQuartoGrafico = await App.Database.GetParametroGlobale(Constants.ConstQuartoGrafico);

            if (pgTerzoGrafico != null)
                IsThirdChartVisible = (pgTerzoGrafico.ValoreParametroBool == true);

            if (pgQuartoGrafico != null)
                IsFourthChartVisible = (pgQuartoGrafico.ValoreParametroBool == true);
        }

        private async Task GetChartData()
        {
            FirstSeriesData = new ObservableCollection<ChartDataModel>();
            SecondSeriesData = new ObservableCollection<ChartDataModel>();
            ThirdSeriesData = new ObservableCollection<ChartDataModel>();
            FourthSeriesData = new ObservableCollection<ChartDataModel>();

            FirstColorCollection = new ObservableCollection<Color>();
            SecondColorCollection = new ObservableCollection<Color>();
            ThirdColorCollection = new ObservableCollection<Color>();
            FourthColorCollection = new ObservableCollection<Color>();

            #region Primo Grafico
            List<ValoriRisposta> valoriRisposta = await App.Database.GetValoriRisposta();
            double totaleDomande = 0;

            Random rnd = new Random();

            foreach (var item in valoriRisposta)
            {
                int contaRisposte = await App.Database.GetCountersAudit(Audit.IdAudit, item.CodValoreRisposta.ToString());
                FirstSeriesData.Add(new ChartDataModel(item.DescValoreRisposta, contaRisposte, ""));
                FirstColorCollection.Add(Color.FromHex(item.ColoreValoreRisposta.Replace("#", "")));

                totaleDomande += contaRisposte;
            }

            int contaRisposteDV = await App.Database.GetCountersAudit(Audit.IdAudit, "");
            totaleDomande += contaRisposteDV;

            ProgressoDomande = ((totaleDomande - contaRisposteDV) * 100) / totaleDomande;

            FirstSeriesData.Add(new ChartDataModel(Resx.AppResources.LabelStatoDV, contaRisposteDV, ""));
            FirstColorCollection.Add(FunctionsHelper.GetColorFromResource("ColoreDV"));
            #endregion

            #region Secondo Grafico
            List<StatiDomandeRisposteDettaglio> statiDomandeRisposteDettaglio = await App.Database.GetStatiNC();

            foreach (var item in statiDomandeRisposteDettaglio)
            {
                //int contaRisposte = await App.Database.GetCountersAuditNc(Audit.IdAudit, null, item.IdStatoNC);
                int contaRisposte = rnd.Next(0, 101);
                SecondSeriesData.Add(new ChartDataModel($"{item.DescStatoNC} / {item.DescStatoEng}", contaRisposte, ""));
                SecondColorCollection.Add(Color.FromHex(item.ColoreStatoNC.Replace("#", "")));
            }
            #endregion

            #region Terzo e Quarto Grafico
            if (IsThirdChartVisible || IsFourthChartVisible)
            {
                List<DatiIndice> items = await App.Database.GetChartData(Audit.IdAudit);

                foreach (var item in items)
                {
                    if (IsThirdChartVisible)
                    {
                        //ThirdSeriesData.Add(new ChartDataModel($"{item.Codice} - {item.Descrizione}", item.Valore, ""));
                        ThirdSeriesData.Add(new ChartDataModel($"{item.Codice} - {item.Descrizione}", rnd.Next(0, 101), ""));
                        ThirdColorCollection.Add(Color.FromHex(item.Colore.Replace("#", "")));
                    }

                    if (IsFourthChartVisible)
                    {
                        //FourthSeriesData.Add(new ChartDataModel($"{item.Codice} - {item.Descrizione}", item.Conteggio, ""));
                        FourthSeriesData.Add(new ChartDataModel($"{item.Codice} - {item.Descrizione}", rnd.Next(0, 101), ""));
                        FourthColorCollection.Add(Color.FromHex(item.Colore.Replace("#", "")));
                    }
                }
            }
            #endregion

            IsViewModelLoaded = true;
        }
    }
}
