﻿using AuditManager.Helpers;
using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.Shared.Types;
using Syncfusion.XForms.DataForm;
using Xamarin.Forms;
using Xamarin.Forms.Internals;

namespace AuditManager.ViewModels
{
    [Preserve(AllMembers = true)]
    public class LoginViewModel : BaseViewModel
    {

        private LoginModel login;
        public LoginModel Login
        {
            get { return login; }
            set { login = value; }
        }

        public Command<object> LoginCommand { get; set; }

        public LoginViewModel()
        {
            Login = new LoginModel();
            IsEnabled = Settings.IsConnected;
            LoginCommand = new Command<object>(OnLogin);

            Settings.IsOnline = true;
        }

        private async void OnLogin(object dataForm)
        {
            MessagingCenter.Send(this, Messages.OperationStarted.ToString());
            IsBusy = true;
            IsEnabled = false;

            var sfDataForm = dataForm as SfDataForm;
            var isValid = sfDataForm.Validate();

            if (!isValid)
            {
                ShowAlert(new AlertModel
                {
                    Title = Resx.AppResources.StringMissingLoginInformationTitle,
                    Description = Resx.AppResources.StringMissingLoginInformationDescription,
                    OkButton = Resx.AppResources.LabelOK
                });
            }
            else
            {
                sfDataForm.Commit();

                LoginModel loginModel = new LoginModel();
                loginModel = FunctionsHelper.Clone(Login);

                ApiResponse<Utenti> loginResponse = await ns.Login(loginModel);

                if (loginResponse.IsSuccess)
                {
                    Settings.User = Login.Email;
                    Settings.Password = Login.Password;
                    Settings.UserName = $"{loginResponse.Response.Nome} {loginResponse.Response.Cognome}";
                    Settings.UserImage = loginResponse.Response.Immagine;
                    Settings.SetTheme(loginResponse.Response.EnumTema);
                    Settings.Theme = (int)loginResponse.Response.EnumTema;

                    ApiResponse<bool> syncDatabase = await SyncDB(true);

                    if (syncDatabase.IsSuccess)
                        Application.Current.MainPage = App.GetMainPage();
                    else
                        ShowAlert(new AlertModel
                        {
                            Title = Resx.AppResources.StringLoginFailedTitle,
                            Description = syncDatabase.ErrorMessage,
                            OkButton = Resx.AppResources.LabelOK
                        });
                }
                else
                {
                    AlertModel alertModel = new AlertModel
                    {
                        Title = Resx.AppResources.StringLoginFailedTitle,
                        OkButton = Resx.AppResources.LabelOK
                    };

                    switch (loginResponse.Status)
                    {
                        case ApiStatus.NoConnectivity:
                            alertModel.Description = Resx.AppResources.StringNoConnectivity;
                            break;
                        case ApiStatus.UnAuthorized:
                            alertModel.Description = Resx.AppResources.StringLoginFailedUnAuthDescription;
                            break;
                        case ApiStatus.Error:
                        case ApiStatus.Exception:
                        case ApiStatus.NotFound:
                        default:
                            alertModel.Description = Resx.AppResources.StringLoginFailedErrDescription;
                            break;
                    }

                    ShowAlert(alertModel);
                }
            }

            IsEnabled = true;
            IsBusy = false;

            MessagingCenter.Send(this, Messages.OperationFinished.ToString());
        }

        private void ShowAlert(AlertModel am)
        {
            AlertButtonOK = am.OkButton;
            AlertTitle = am.Title;
            AlertText = am.Description;

            IsOpenAlert = true;
            MessagingCenter.Send(this, Messages.DisplayAlertMessage.ToString(), am);
        }
    }
}
