﻿using AuditManager.Data;
using AuditManager.Helpers;
using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.Shared.Constants;
using AuditManager.Shared.Extensions;
using AuditManager.Views;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace AuditManager.ViewModels
{
    public class QuestionarioViewModel : BaseViewModel
    {
        #region Properties
        private Audit audit;
        public Audit Audit
        {
            get { return audit; }
            set
            {
                audit = value;
                OnPropertyChanged(nameof(Audit));
            }
        }

        private AuditCategorie categoria;
        public AuditCategorie Categoria
        {
            get { return categoria; }
            set
            {
                categoria = value;
                OnPropertyChanged(nameof(Categoria));
            }
        }

        private ObservableCollection<AuditDomandeRisposteXF> listaDomande;
        public ObservableCollection<AuditDomandeRisposteXF> ListaDomande
        {
            get { return listaDomande; }
            set
            {
                listaDomande = value;
                OnPropertyChanged(nameof(ListaDomande));
            }
        }

        private bool isOpen = false;
        public bool IsOpen
        {
            get { return isOpen; }
            set
            {
                bool oldValue = isOpen;
                isOpen = value;
                OnPropertyChanged(nameof(IsOpen));

                if (oldValue != isOpen && !isOpen)
                    MessagingCenter.Send(this, Messages.ClosedFilterMessage.ToString(), "F");
            }
        }

        private bool aggiungiAllegatoVisible = false;
        public bool AggiungiAllegatoVisible
        {
            get { return aggiungiAllegatoVisible; }
            set
            {
                aggiungiAllegatoVisible = value;
                OnPropertyChanged(nameof(AggiungiAllegatoVisible));
            }
        }
        #endregion

        #region Commands
        public Command BackCommand { get; set; }
        public Command AddCommand { get; set; }

        public Command<string> OpenFiltersCommand { get; set; }
        public Command ToggleFilterCommand { get; set; }
        public Command<string> ToggleFilterListCommand { get; set; }

        public Command<object> NavigateCommand { get; set; }

        public Command RispostaMultiplaCommand { get; set; }

        public Command<object> AggiungiAllegatoCommand { get; set; }
        public Command<object> ListaAllegatiCommand { get; set; }
        public Command<object> AggiungiNCCommand { get; set; }
        public Command<object> ListaNCCommand { get; set; }
        public Command<object> AzzeraRispostaCommand { get; set; }
        public Command<object> HelpCommand { get; set; }
        public Command<object> ProcessoCommand { get; set; }
        public Command<object> CancellaDomandaCommand { get; set; }
        #endregion


        public QuestionarioViewModel() : base()
        {

        }

        public QuestionarioViewModel(NavigationParameterModel navigationParameter) : base()
        {
            IsVisible = true;
            IsBusy = false;
            IsOnline = Settings.IsOnline;

            AddVisible = false;
            SearchVisible = false;

            Audit = navigationParameter.Audit;
            Categoria = navigationParameter.Categoria;

            Audit.BloccoApp1 = Audit.BloccoApp1.ReplaceNewlines(" - ");
            Audit.BloccoApp4 = $"{Audit.BloccoApp2} - {Audit.BloccoApp3}";
            Audit.BloccoApp4 = Audit.BloccoApp4.ReplaceNewlines(" - ");

            listaDomande = new ObservableCollection<AuditDomandeRisposteXF>();
            ListaDomande = new ObservableCollection<AuditDomandeRisposteXF>();

            DettaglioAuditCommand = new Command(async () => await ExecuteDettaglioAuditCommand());
            AllegatoPrincipaleCommand = new Command(async () => await ExecuteAllegatoPrincipaleCommand());
            CompletaCommand = new Command(async () => await ExecuteCompletaCommand());
            StampaCommand = new Command(async () => await ExecuteStampaCommand());
            ListaLavoriCommand = new Command(async () => await ExecuteListaLavoriCommand());
            NotificaListaLavoriCommand = new Command(async () => await ExecuteNotificaListaLavoriCommand());
            ConvalidaCommand = new Command(async () => await ExecuteConvalidaCommand());
            SyncCommand = new Command(async () => await ExecuteSyncCommand());

            ToggleFilterCommand = new Command(() => ExecuteToggleFilterCommand());
            OpenFiltersCommand = new Command<string>((item) => ExecuteOpenFiltersCommand(item));
            ToggleFilterListCommand = new Command<string>((s) => ExecuteToggleFilterListCommand(s));

            NavigateCommand = new Command<object>(async (s) => await ExecuteNavigateCommand(s));

            BackCommand = new Command(async () => await ExecuteBackCommand());
            AddCommand = new Command(async () => await ExecuteAddCommand());
            RispostaMultiplaCommand = new Command(async () => await ExecuteRispostaMultiplaCommand());

            AggiungiAllegatoCommand = new Command<object>(async (obj) => await ExecuteAggiungiAllegatoCommand(obj));
            ListaAllegatiCommand = new Command<object>(async (obj) => await ExecuteListaAllegatiCommand(obj));
            AggiungiNCCommand = new Command<object>(async (obj) => await ExecuteAggiungiNCCommand(obj));
            ListaNCCommand = new Command<object>(async (obj) => await ExecuteListaNCCommand(obj));
            AzzeraRispostaCommand = new Command<object>(async (obj) => await ExecuteAzzeraRispostaCommand(obj));
            HelpCommand = new Command<object>(async (obj) => await ExecuteHelpCommand(obj));
            ProcessoCommand = new Command<object>(async (obj) => await ExecuteProcessoCommand(obj));
            CancellaDomandaCommand = new Command<object>(async (obj) => await ExecuteCancellaDomandaCommand(obj));

            Task.Run(async () =>
            {
                await GetDomande();

                AllegatoPrincipaleVisible = true;
                IsEnabled = (Audit.IdStato < 4 && !Categoria.Readonly);

                List<AuditTipiAudit> listaTipiAudit = await App.Database.GetAuditTipiAudit(Audit.IdAudit);

                foreach (var item in listaTipiAudit)
                {
                    if (!FilterVisible)
                        FilterVisible = await App.Database.GetFunzioniApp(29, item.IdTipoAudit);

                    if (!AddDomandaVisible)
                        AddDomandaVisible = await App.Database.GetFunzioniApp(7, item.IdTipoAudit);

                    if (!RispostaMultiplaVisible)
                        RispostaMultiplaVisible = await App.Database.GetFunzioniApp(8, item.IdTipoAudit);

                    if (!DettaglioAuditVisible)
                        DettaglioAuditVisible = await App.Database.GetFunzioniApp(3, item.IdTipoAudit);

                    if (!StampaVisible)
                        StampaVisible = await App.Database.GetFunzioniApp(4, item.IdTipoAudit);

                    if (!ListaLavoriVisible)
                        ListaLavoriVisible = await App.Database.GetFunzioniApp(5, item.IdTipoAudit);

                    if (!NotificaListaLavoriVisible)
                        NotificaListaLavoriVisible = await App.Database.GetFunzioniApp(6, item.IdTipoAudit);

                    if (!ConvalidaVisible)
                        ConvalidaVisible = await App.Database.GetFunzioniApp(39, item.IdTipoAudit);

                    if (!AggiungiAllegatoVisible)
                        AggiungiAllegatoVisible = (await App.Database.GetFunzioniApp(9, item.IdTipoAudit) && Audit.IdStato < 4 && !Categoria.Readonly);
                }

                if (IsVisiblePhone)
                {
                    MoreButtonsVisible = true;
                    HeightPopUp = 80;
                }

                if (IsVisiblePhone && DettaglioAuditVisible)
                    HeightPopUp += 40;

                if (IsVisiblePhone && AllegatoPrincipaleVisible)
                    HeightPopUp += 40;

                if (RispostaMultiplaVisible)
                    HeightPopUp += 40;

                if (CompletaVisible)
                    HeightPopUp += 40;

                if (StampaVisible)
                    HeightPopUp += 40;

                if (ListaLavoriVisible)
                    HeightPopUp += 40;

                if (NotificaListaLavoriVisible)
                    HeightPopUp += 40;

                if (ConvalidaVisible)
                    HeightPopUp += 40;
            });
        }

        public async Task GetDomande()
        {
            List<AuditDomandeRisposte> lista = await App.Database.GetAuditDomande(Audit.IdAudit, Categoria.IdCategoria);
            List<AuditDomandeRisposteXF> listaReturn = new List<AuditDomandeRisposteXF>();

            foreach (var item in lista)
            {
                AuditDomandeRisposteXF itemXF = new AuditDomandeRisposteXF(item);

                if (itemXF.IdDomandaParent == null)
                    itemXF.IsDomandaVisibile = true;
                else
                {
                    AuditDomandeRisposte d = await App.Database.GetAuditDomandaFromParent(itemXF.IdAudit, itemXF.IdDomandaParent);
                    itemXF.IsDomandaVisibile = (d.Risposta == itemXF.ValoreDomandaParent);
                }

                //itemXF.NrAllegati = await App.Database.ContaAllegatiDomanda(itemXF.IdAudit, itemXF.IdRisposta);
                //itemXF.NrDettagli = await App.Database.ContaDettagliDomanda(itemXF.IdAudit, itemXF.IdRisposta);

                //itemXF.ListaAllegatiVisible = (itemXF.NrAllegati > 0);
                //itemXF.AggiungiNCVisible = (itemXF.Attributo == 1 && Audit.IdStato < 4 && !Categoria.Readonly);
                //itemXF.ListaNCVisible = (itemXF.NrDettagli > 0);
                //itemXF.AzzeraRispostaVisible = (!string.IsNullOrEmpty(itemXF.Risposta) && Audit.IdStato < 4 && !Categoria.Readonly);
                //itemXF.CancellaDomandaVisible = (itemXF.DomandaCustom && Audit.IdStato < 4 && !Categoria.Readonly);

                //if (AggiungiAllegatoVisible)
                //    itemXF.HeightPopUpDomanda += 40;

                //if (itemXF.ListaAllegatiVisible)
                //    itemXF.HeightPopUpDomanda += 40;

                //if (itemXF.AggiungiNCVisible)
                //    itemXF.HeightPopUpDomanda += 40;

                //if (itemXF.ListaNCVisible)
                //    itemXF.HeightPopUpDomanda += 40;

                //if (itemXF.AzzeraRispostaVisible)
                //    itemXF.HeightPopUpDomanda += 40;

                //if (itemXF.HelpVisible)
                //    itemXF.HeightPopUpDomanda += 40;

                //if (itemXF.CancellaDomandaVisible)
                //    itemXF.HeightPopUpDomanda += 40;

                itemXF.ColoreRisposta = FunctionsHelper.GetColoreRisposta(itemXF.Risposta, itemXF.IdTipoDomanda);

                itemXF.ColoreBackgroundBottoneOK = (itemXF.Risposta == "1") ? Constants.ColoreRispostaOK : "";
                itemXF.ColoreTestoBottoneOK = (itemXF.Risposta == "1") ? Constants.ColoreBianco : Constants.ColoreRispostaOK;
                itemXF.ColoreBackgroundBottoneKO = (itemXF.Risposta == "0") ? Constants.ColoreRispostaKO : "";
                itemXF.ColoreTestoBottoneKO = (itemXF.Risposta == "0") ? Constants.ColoreBianco : Constants.ColoreRispostaKO;
                itemXF.ColoreBackgroundBottoneNA = (itemXF.Risposta == "2") ? Constants.ColoreRispostaNA : "";
                itemXF.ColoreTestoBottoneNA = (itemXF.Risposta == "2") ? Constants.ColoreBianco : Constants.ColoreRispostaNA;
                itemXF.ColoreBackgroundBottoneNT = (itemXF.Risposta == "3") ? Constants.ColoreRispostaNT : "";
                itemXF.ColoreTestoBottoneNT = (itemXF.Risposta == "3") ? Constants.ColoreBianco : Constants.ColoreRispostaNT;
                
                itemXF.RispostaParametroKO = new RispostaParametro(Audit.IdAudit, Categoria.IdCategoria, itemXF.IdDomanda, itemXF.IdTipoDomanda, itemXF.IdRisposta, itemXF.Attributo, "0");
                itemXF.RispostaParametroOK = new RispostaParametro(Audit.IdAudit, Categoria.IdCategoria, itemXF.IdDomanda, itemXF.IdTipoDomanda, itemXF.IdRisposta, itemXF.Attributo, "1");
                itemXF.RispostaParametroNA = new RispostaParametro(Audit.IdAudit, Categoria.IdCategoria, itemXF.IdDomanda, itemXF.IdTipoDomanda, itemXF.IdRisposta, itemXF.Attributo, "2");
                itemXF.RispostaParametroNT = new RispostaParametro(Audit.IdAudit, Categoria.IdCategoria, itemXF.IdDomanda, itemXF.IdTipoDomanda, itemXF.IdRisposta, itemXF.Attributo, "3");
                itemXF.RispostaParametro = new RispostaParametro(Audit.IdAudit, Categoria.IdCategoria, itemXF.IdDomanda, itemXF.IdTipoDomanda, itemXF.IdRisposta, itemXF.Attributo, item.Risposta);

                itemXF.Domanda1Visibile = (itemXF.IdTipoDomanda == 1);
                itemXF.Domanda2Visibile = (itemXF.IdTipoDomanda == 2);
                itemXF.Domanda3Visibile = (itemXF.IdTipoDomanda == 3);
                itemXF.Domanda4Visibile = (itemXF.IdTipoDomanda == 4);
                itemXF.Domanda5Visibile = (itemXF.IdTipoDomanda == 5);
                itemXF.Domanda6Visibile = (itemXF.IdTipoDomanda == 6);
                itemXF.Domanda7Visibile = (itemXF.IdTipoDomanda == 7);
                itemXF.Domanda8Visibile = (itemXF.IdTipoDomanda == 8);
                itemXF.Domanda9Visibile = (itemXF.IdTipoDomanda == 9);
                itemXF.Domanda10Visibile = (itemXF.IdTipoDomanda == 10);
                itemXF.Domanda11Visibile = (itemXF.IdTipoDomanda == 11);

                itemXF.RispostaCommand = new Command<RispostaParametro>(async (obj) => await ExecuteRispostaCommand(obj));
                itemXF.IsEnabled = IsEnabled;

                listaReturn.Add(itemXF);
            }

            ListaDomande = new ObservableCollection<AuditDomandeRisposteXF>(listaReturn);
        }

        #region Command
        private void ExecuteToggleFilterCommand()
        {
            IsOpen = !IsOpen;
            IsOpenFilters = IsOpen;
        }

        private void ExecuteOpenFiltersCommand(string item)
        {
            IsOpenFiltersList = true;
            FilterHeight = 350;

            IsVisibleListTipiAudit = false;
            IsVisibleListSoggetti = false;
            IsVisibleListMotivazioni = false;
            IsVisibleListaValoriRisposte = true;

            LabelFilter = item.ToTitleCase();
        }

        private void ExecuteToggleFilterListCommand(object s)
        {
            if (s.Equals("R"))
            {
                foreach (var item in ListaValoriRisposte.Where(w => w.IsSelected == true).ToList())
                    item.IsSelected = false;
            }

            ListaValoriRisposteSelezionati = new ObservableCollection<ValoriRisposte>(ListaValoriRisposte.Where(w => w.IsSelected == true));

            if (s.Equals("F"))
            {
                IsOpen = !IsOpen;
                IsOpenFilters = false;
            }

            IsOpenFiltersList = false;
            FilterHeight = ConstFilterHeight;
        }

        private async Task ExecuteBackCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteBackCommand), string.Empty);

            await App.NavigationService.NavigateAsync(nameof(CategoriePage), Audit);
        }

        private async Task ExecuteSyncCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteSyncCommand), string.Empty);

            await SyncDB();
            await GetDomande();
        }

        private async Task ExecuteAllegatoPrincipaleCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteAllegatoPrincipaleCommand), string.Empty);
        }

        private async Task ExecuteDettaglioAuditCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteDettaglioAuditCommand), string.Empty);

            //await App.NavigationService.NavigateAsync("DettaglioAuditPage", Audit);
        }

        private async Task ExecuteStampaCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteStampaCommand), string.Empty);

            //await App.NavigationService.NavigateAsync("StampaAuditPage", Audit);
        }

        private async Task ExecuteConvalidaCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteConvalidaCommand), string.Empty);

            //await App.NavigationService.NavigateAsync("ConvalidaAuditPage", Audit);
        }

        private async Task ExecuteListaLavoriCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteListaLavoriCommand), string.Empty);

            //await App.NavigationService.NavigateAsync("ListaLavoriPage", Audit);
        }

        private async Task ExecuteNotificaListaLavoriCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteNotificaListaLavoriCommand), string.Empty);
        }

        private async Task ExecuteCompletaCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteCompletaCommand), string.Empty);

            //await App.NavigationService.NavigateAsync("CompletaAuditPage", Audit);
        }

        private async Task ExecuteNavigateCommand(object obj)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteNavigateCommand), string.Empty);
        }

        private async Task ExecuteAddCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteAddCommand), string.Empty);
        }

        private async Task ExecuteRispostaMultiplaCommand()
        {
            await FunctionsHelper.Log(nameof(this.ExecuteRispostaMultiplaCommand), string.Empty);
        }

        private async Task ExecuteCancellaDomandaCommand(object s)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteCancellaDomandaCommand), string.Empty);
        }

        private async Task ExecuteProcessoCommand(object obj)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteProcessoCommand), string.Empty);
        }

        private async Task ExecuteHelpCommand(object obj)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteHelpCommand), string.Empty);
        }

        private async Task ExecuteAzzeraRispostaCommand(object obj)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteAzzeraRispostaCommand), string.Empty);
        }

        private async Task ExecuteListaNCCommand(object obj)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteListaNCCommand), string.Empty);
        }

        private async Task ExecuteAggiungiNCCommand(object obj)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteAggiungiNCCommand), string.Empty);
        }

        private async Task ExecuteListaAllegatiCommand(object obj)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteListaAllegatiCommand), string.Empty);
        }

        private async Task ExecuteAggiungiAllegatoCommand(object obj)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteAggiungiAllegatoCommand), string.Empty);
        }

        private async Task ExecuteRispostaCommand(RispostaParametro obj)
        {
            await FunctionsHelper.Log(nameof(this.ExecuteRispostaCommand), string.Empty);

            if (obj != null)
            {
                AuditDomandeRisposteXF item = ListaDomande.Where(w => w.IdDomanda == obj.IdDomanda).FirstOrDefault();
                item.ColoreRisposta = FunctionsHelper.GetColoreRisposta(obj.Risposta, obj.IdTipoDomanda);
                item.Risposta = obj.Risposta;
                item.AzzeraRispostaVisible = (!string.IsNullOrEmpty(obj.Risposta));

                item.ColoreBackgroundBottoneOK = "";
                item.ColoreTestoBottoneOK = Constants.ColoreRispostaOK;
                item.ColoreBackgroundBottoneKO = "";
                item.ColoreTestoBottoneKO = Constants.ColoreRispostaKO;
                item.ColoreBackgroundBottoneNA = "";
                item.ColoreTestoBottoneNA = Constants.ColoreRispostaNA;
                item.ColoreBackgroundBottoneNT = "";
                item.ColoreTestoBottoneNT = Constants.ColoreRispostaNT;

                switch (obj.Risposta)
                {
                    case "0":
                        item.ColoreBackgroundBottoneKO = Constants.ColoreRispostaKO;
                        item.ColoreTestoBottoneKO = Constants.ColoreBianco;
                        break;
                    case "1":
                        item.ColoreBackgroundBottoneOK = Constants.ColoreRispostaOK;
                        item.ColoreTestoBottoneOK = Constants.ColoreBianco;
                        break;
                    case "2":
                        item.ColoreBackgroundBottoneNA = Constants.ColoreRispostaNA;
                        item.ColoreTestoBottoneNA = Constants.ColoreBianco;
                        break;
                    case "3":
                        item.ColoreBackgroundBottoneNT = Constants.ColoreRispostaNT;
                        item.ColoreTestoBottoneNT = Constants.ColoreBianco;
                        break;
                    default:
                        break;
                }

                await App.Database.UpdateAuditDomandeRisposte(item);

                if (Audit.IdStato != 3)
                    await FunctionsHelper.UpdateStatoAudit(Audit, 3);
            }
        }

        //private async Task ExecuteRispostaCommand1(object obj)
        //{
        //    await FunctionsHelper.Log(nameof(this.ExecuteRispostaCommand1), string.Empty);

        //    if (obj != null)
        //    {
                
        //    }
        //}
        #endregion
    }
}