﻿using AuditManager.Behaviors;
using AuditManager.Converters;
using AuditManager.Helpers;
using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.Shared.Extensions;
using AuditManager.ViewModels;
using Syncfusion.DataSource;
using Syncfusion.SfNavigationDrawer.XForms;
using Syncfusion.XForms.TabView;
using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace AuditManager.Views
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class AuditListPage : BasePage
    {
        private AuditListViewModel Modello
        {
            get { return BindingContext as AuditListViewModel; }
        }

        public AuditListPage() : base()
        {
            try
            {
                InitializeComponent();
                hamburgerButton.Image = (FileImageSource)FunctionsHelper.SetImageSource("icon_menu_light.png");

                CreateListView();
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        protected override void OnAppearing()
        {
            base.OnAppearing();

            MessagingCenter.Subscribe<AuditListViewModel, AlertModel>(this, Messages.DisplayAlertMessage.ToString(), (sender, alertValues) =>
            {
                AlertVibrate(alertValues);
            });

            MessagingCenter.Subscribe<AuditListViewModel>(this, Messages.HomePage.ToString(), (sender) =>
            {
                Application.Current.MainPage = App.GetMainPage();
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpOffline.ToString(), (sender) =>
            {
                Modello.OfflineCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpSync.ToString(), async (sender) =>
            {
                topHeaderBar.Start();
                Modello.SyncCommand.Execute(null);
                topHeaderBar.Stop();
            });

            MessagingCenter.Subscribe<AuditListViewModel, string>(this, Messages.ClosedFilterMessage.ToString(), async (sender, s) =>
            {
                if (s.Equals("F"))
                {
                    navigationDrawer.ToggleSecondaryDrawer();

                    await Modello.LoadData(false);

                    if (Device.Idiom == TargetIdiom.Phone)
                        CreateListView(true);
                }
            });
        }

        protected override void OnDisappearing()
        {
            base.OnDisappearing();

            MessagingCenter.Unsubscribe<AuditListViewModel, AlertModel>(this, Messages.DisplayAlertMessage.ToString());
            MessagingCenter.Unsubscribe<AuditListViewModel>(this, Messages.HomePage.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpOffline.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpSync.ToString());
            MessagingCenter.Unsubscribe<AuditListViewModel, string>(this, Messages.ClosedFilterMessage.ToString());
        }

        private void CreateListView(bool reCreate = false)
        {
            if (reCreate)
            {
                foreach (var item in sfTabView.Items)
                {
                    CustomTabItem stato = (CustomTabItem)item.HeaderContent;

                    if (stato != null)
                    {
                        ListView listView = (ListView)item.Content;
                        listView.ItemsSource = new ObservableCollection<Audit>(Modello.Audits.Where(w => w.IdStato == stato.IdStato).ToList());
                        item.Content = listView;
                    }
                }
            }
            else
            {
                int stato = 0;

                foreach (var item in Modello.ListaStati)
                {
                    ObservableCollection<Audit> listaAudit = new ObservableCollection<Audit>(Modello.Audits.Where(w => w.IdStato == item.IdStato).ToList());

                    ListView listView = new ListView
                    {
                        HasUnevenRows = true,
                        RowHeight = -1,
                        ItemsSource = listaAudit,
                        BackgroundColor = (Color)Application.Current.Resources["AppBackgroundColor"],
                        ItemTemplate = new DataTemplate(typeof(AuditView)),
                        HorizontalOptions = LayoutOptions.FillAndExpand,
                        VerticalOptions = LayoutOptions.FillAndExpand,
                        SeparatorVisibility = SeparatorVisibility.None,
                        SeparatorColor = Color.Transparent
                    };

                    listView.Behaviors.Add(new EventToCommandBehavior
                    {
                        Command = ((AuditListViewModel)BindingContext).SelectedItemCommand,
                        Converter = new ItemTappedEventArgsToItemTappedConverter(),
                        EventName = "ItemTapped"
                    });

                    CustomTabItem customTabItem = new CustomTabItem
                    {
                        Immagine = item.Immagine,
                        Title = item.DescStato.ToUpper(),
                        GridId = stato,
                        IdStato  = item.IdStato
                    };

                    if (stato == Modello.TabItemSelezionato)
                        customTabItem.Immagine = customTabItem.Immagine.ReplaceWithNullControl("disabled", "light");

                    TapGestureRecognizer tabGesture = new TapGestureRecognizer();
                    tabGesture.Tapped += TabGesture_Tapped;
                    customTabItem.GestureRecognizers.Add(tabGesture);

                    sfTabView.Items.Add(new SfTabItem()
                    {
                        Title = customTabItem.Title,
                        BindingContext = item,
                        Content = listView,
                        HeaderContent = customTabItem,
                        SelectionColor = (Color)Application.Current.Resources["LoginButtonColor"]
                    });

                    stato++;
                }
            }
        }

        private async void ListView_Loaded(object sender, Syncfusion.ListView.XForms.ListViewLoadedEventArgs e)
        {
            await Task.Delay(2000);
            listViewTablet.RefreshView();
        }

        private void ListView_ItemTapped(object sender, ItemTappedEventArgs e)
        {
            DisplayAlert("ListView_ItemTapped Event", "item Tapped", "ok");
        }

        private void TabGesture_Tapped(object sender, EventArgs e)
        {
            if (sender is CustomTabItem)
            {
                CustomTabItem customTabItem = sender as CustomTabItem;

                foreach (var item in sfTabView.Items.Where(s => s.HeaderContent != customTabItem).ToList())
                {
                    string immagine = (item.HeaderContent as CustomTabItem).Immagine;
                    (item.HeaderContent as CustomTabItem).Immagine = immagine.ReplaceWithNullControl("light", "disabled");
                }

                Modello.TabItemSelezionato = customTabItem.GridId;
                customTabItem.Immagine = customTabItem.Immagine.ReplaceWithNullControl("disabled", "light");
            }
        }
    }
}