﻿using AuditManager.Models;
using AuditManager.Resx;
using AuditManager.Services;
using AuditManager.Shared.Constants;
using Plugin.Permissions;
using Plugin.Permissions.Abstractions;
using Plugin.Vibrate;
using System;
using System.Threading.Tasks;
using Xamarin.Forms;
using Xamarin.Forms.Internals;
using Xamarin.Forms.Xaml;

namespace AuditManager.Views
{
    [Preserve(AllMembers = true)]
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class BasePage : ContentPage
    {
        #region Props
        public NetworkService Ns { get; set; }
        #endregion

        public BasePage()
        {
            CheckPermissionStatus();
            Ns = new NetworkService();
            NavigationPage.SetHasNavigationBar(this, false);
        }

        public async Task AlertError(AlertModel model)
        {
            await DisplayAlert(model.Title, model.Description, model.OkButton);
        }

        public void AlertVibrate(AlertModel values)
        {
            var v = CrossVibrate.Current;
            v.Vibration(Constants.VibrateTime);

            //await DisplayAlert(values.Title, values.Description, values.OkButton);
        }

        public async void CheckPermissionStatus()
        {
            var status = await CrossPermissions.Current.CheckPermissionStatusAsync(Permission.Storage);

            if (status != PermissionStatus.Granted)
            {
                AlertModel alertModel = new AlertModel
                {
                    Title = AppResources.LabelNeedStorageTitle,
                    Description = AppResources.LabelNeedStorageDescription,
                    OkButton = AppResources.LabelOK
                };

                if (await CrossPermissions.Current.ShouldShowRequestPermissionRationaleAsync(Permission.Storage))
                    await AlertError(alertModel);

                var results = await CrossPermissions.Current.RequestPermissionsAsync(Permission.Storage);

                if (results.ContainsKey(Permission.Storage))
                    status = results[Permission.Storage];
            }
        }

        //internal T FindByName<T>(string v)
        //{
        //    throw new NotImplementedException();
        //}

        //public async Task Logout(ConfirmModel confirmValues)
        //{
        //    bool answer = await DisplayAlert(confirmValues.Title, confirmValues.Description, confirmValues.OkButton, confirmValues.KoButton);

        //    if (answer)
        //        FunctionsHelper.Logout();
        //}
    }
}
