﻿using AuditManager.Behaviors;
using AuditManager.Helpers;
using AuditManager.Models;
using AuditManager.Models.Tables;
using AuditManager.ViewModels;
using System;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace AuditManager.Views
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class CategoriePage : BasePage
    {
        private CategorieViewModel Modello
        {
            get { return BindingContext as CategorieViewModel; }
        }


        public CategoriePage(Audit item)
        {
            try
            {
                InitializeComponent();

                Behaviors.Add(new CategorieBehavior(item));
                chartGridView.Behaviors.Add(new ChartGridViewBehavior(item));

                backButton.Image = (FileImageSource)FunctionsHelper.SetImageSource("icon_back_arrow_light.png");
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        protected override void OnAppearing()
        {
            base.OnAppearing();

            MessagingCenter.Subscribe<CategorieViewModel, AlertModel>(this, Messages.DisplayAlertMessage.ToString(), (sender, alertValues) =>
            {
                AlertVibrate(alertValues);
            });

            MessagingCenter.Subscribe<CategorieViewModel>(this, Messages.HomePage.ToString(), (sender) =>
            {
                Application.Current.MainPage = App.GetMainPage();
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpOffline.ToString(), (sender) =>
            {
                Modello.OfflineCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpSync.ToString(), (sender) =>
            {
                topHeaderBar.Start();
                Modello.SyncCommand.Execute(null);
                topHeaderBar.Stop();
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpCompleta.ToString(), (sender) =>
            {
                Modello.CompletaCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpListaLavori.ToString(), (sender) =>
            {
                Modello.ListaLavoriCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpConvalida.ToString(), (sender) =>
            {
                Modello.ConvalidaCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpStampa.ToString(), (sender) =>
            {
                Modello.StampaCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpNotificaLista.ToString(), (sender) =>
            {
                Modello.NotificaListaLavoriCommand.Execute(null);
            });

            MessagingCenter.Subscribe<CategorieViewModel, string>(this, Messages.ClosedFilterMessage.ToString(), async (sender, s) =>
            {
                if (s.Equals("F"))
                {
                    navigationDrawer.ToggleDrawer();
                    Modello.GetCategorie();
                }
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpDettaglio.ToString(), (sender) =>
            {
                Modello.DettaglioAuditCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpAllegatoPrincipale.ToString(), (sender) =>
            {
                Modello.AllegatoPrincipaleCommand.Execute(null);
            });
        }

        protected override void OnDisappearing()
        {
            base.OnDisappearing();

            MessagingCenter.Unsubscribe<CategorieViewModel, AlertModel>(this, Messages.DisplayAlertMessage.ToString());
            MessagingCenter.Unsubscribe<CategorieViewModel>(this, Messages.HomePage.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpOffline.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpSync.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpCompleta.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpListaLavori.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpConvalida.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpStampa.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpNotificaLista.ToString());
            MessagingCenter.Unsubscribe<CategorieViewModel, string>(this, Messages.ClosedFilterMessage.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpDettaglio.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpAllegatoPrincipale.ToString());
        }
    }
}