﻿using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace AuditManager.Views
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class ChartGridView : ContentView
    {
        public static readonly BindableProperty ImageProperty = BindableProperty.Create(
            propertyName: nameof(Image),
            returnType: typeof(string),
            declaringType: typeof(ChartGridView),
            defaultValue: string.Empty,
            defaultBindingMode: BindingMode.TwoWay,
            propertyChanged: ImagePropertyChanged);

        public string Image { get { return GetValue(ImageProperty).ToString(); } set { SetValue(ImageProperty, value); } }

        private static void ImagePropertyChanged(BindableObject bindable, object oldValue, object newValue)
        {
            var control = (ChartGridView)bindable;
            control.centerImage.Source = newValue.ToString();
        }

        public ChartGridView()
        {
            InitializeComponent();

            if (Device.Idiom == TargetIdiom.Phone)
            {
                Grid.SetRow(FirstStackChart, 0);
                Grid.SetColumn(FirstStackChart, 0);
                Grid.SetColumnSpan(FirstStackChart, 2);

                Grid.SetRow(SecondStackChart, 1);
                Grid.SetColumn(SecondStackChart, 0);
                Grid.SetColumnSpan(SecondStackChart, 2);

                Grid.SetRow(ThirdStackChart, 2);
                Grid.SetColumn(ThirdStackChart, 0);
                Grid.SetColumnSpan(ThirdStackChart, 2);

                Grid.SetRow(FourthStackChart, 3);
                Grid.SetColumn(FourthStackChart, 0);
                Grid.SetColumnSpan(FourthStackChart, 2);
            }

            if (Device.Idiom == TargetIdiom.Tablet || Device.Idiom == TargetIdiom.Desktop)
            {
                Grid.SetRow(FirstStackChart, 0);
                Grid.SetColumn(FirstStackChart, 0);
                Grid.SetColumnSpan(FirstStackChart, 1);

                Grid.SetRow(SecondStackChart, 0);
                Grid.SetColumn(SecondStackChart, 1);
                Grid.SetColumnSpan(SecondStackChart, 1);

                Grid.SetRow(ThirdStackChart, 1);
                Grid.SetColumn(ThirdStackChart, 0);
                Grid.SetColumnSpan(ThirdStackChart, 1);

                Grid.SetRow(FourthStackChart, 1);
                Grid.SetColumn(FourthStackChart, 1);
                Grid.SetColumnSpan(FourthStackChart, 1);
            }
        }

        protected override void OnSizeAllocated(double width, double height)
        {
            base.OnSizeAllocated(width, height);
        }
    }
}