﻿using AuditManager.Helpers;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

namespace AuditManager.Views
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class MenuItem : ContentView
    {
        public static readonly BindableProperty TestoProperty = BindableProperty.Create(
           propertyName: nameof(Testo),
           returnType: typeof(string),
           declaringType: typeof(MenuItem),
           defaultValue: string.Empty,
           defaultBindingMode: BindingMode.TwoWay,
           propertyChanged: TestoPropertyChanged);

        public string Testo { get { return GetValue(TestoProperty).ToString(); } set { SetValue(TestoProperty, value); } }

        private static void TestoPropertyChanged(BindableObject bindable, object oldValue, object newValue)
        {
            var control = (MenuItem)bindable;
            control.label.Text = newValue.ToString();
        }

        public static readonly BindableProperty ImmagineProperty = BindableProperty.Create(
            propertyName: nameof(Immagine),
            returnType: typeof(string),
            declaringType: typeof(MenuItem),
            defaultValue: string.Empty,
            defaultBindingMode: BindingMode.TwoWay,
            propertyChanged: ImageSourcePropertyChanged);

        public string Immagine { get { return GetValue(ImmagineProperty).ToString(); } set { SetValue(ImmagineProperty, value); } }

        private static void ImageSourcePropertyChanged(BindableObject bindable, object oldValue, object newValue)
        {
            var control = (MenuItem)bindable;
            control.image.Source = FunctionsHelper.SetImageSource(newValue.ToString());
        }

        public static readonly BindableProperty ColoreProperty = BindableProperty.Create(
            propertyName: nameof(Colore),
            returnType: typeof(Color),
            declaringType: typeof(MenuItem),
            defaultValue: Application.Current.Resources["DarkTextColor"],
            defaultBindingMode: BindingMode.TwoWay,
            propertyChanged: ColorePropertyChanged);

        public Color Colore { get { return (Color)GetValue(ColoreProperty); } set { SetValue(ColoreProperty, value); } }

        private static void ColorePropertyChanged(BindableObject bindable, object oldValue, object newValue)
        {
            var control = (MenuItem)bindable;
            control.label.TextColor = (Color)newValue;
        }



        public MenuItem()
        {
            InitializeComponent();
        }
    }
}