﻿using AuditManager.Helpers;
using AuditManager.Models;
using AuditManager.ViewModels;
using System;
using Xamarin.Forms;
using Xamarin.Forms.Internals;
using Xamarin.Forms.Xaml;

namespace AuditManager.Views
{
    [Preserve(AllMembers = true)]
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class QuestionarioPage : BasePage
    {
        private QuestionarioViewModel Modello
        {
            get { return BindingContext as QuestionarioViewModel; }
        }


        public QuestionarioPage()
        {
            InitializeComponent();
        }

        public QuestionarioPage(NavigationParameterModel parametro)
        {
            try
            {
                InitializeComponent();
                BindingContext = new QuestionarioViewModel(parametro);
                //Behaviors.Add(new QuestionarioBehavior(NavigationParameterModel parametro));

                backButton.Image = (FileImageSource)FunctionsHelper.SetImageSource("icon_back_arrow_light.png");
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        protected override void OnAppearing()
        {
            base.OnAppearing();

            MessagingCenter.Subscribe<QuestionarioViewModel, AlertModel>(this, Messages.DisplayAlertMessage.ToString(), (sender, alertValues) =>
            {
                AlertVibrate(alertValues);
            });

            MessagingCenter.Subscribe<QuestionarioViewModel>(this, Messages.HomePage.ToString(), (sender) =>
            {
                Application.Current.MainPage = App.GetMainPage();
            });

            MessagingCenter.Subscribe<QuestionarioViewModel, string>(this, Messages.ClosedFilterMessage.ToString(), async (sender, s) =>
            {
                if (s.Equals("F"))
                    navigationDrawer.ToggleDrawer();
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpOffline.ToString(), (sender) =>
            {
                Modello.OfflineCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpSync.ToString(), (sender) =>
            {
                topHeaderBar.Start();
                Modello.SyncCommand.Execute(null);
                topHeaderBar.Stop();
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpCompleta.ToString(), (sender) =>
            {
                Modello.CompletaCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpListaLavori.ToString(), (sender) =>
            {
                Modello.ListaLavoriCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpConvalida.ToString(), (sender) =>
            {
                Modello.ConvalidaCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpStampa.ToString(), (sender) =>
            {
                Modello.StampaCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpNotificaLista.ToString(), (sender) =>
            {
                Modello.NotificaListaLavoriCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpDettaglio.ToString(), (sender) =>
            {
                Modello.DettaglioAuditCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpRispostaMultipla.ToString(), (sender) =>
            {
                Modello.RispostaMultiplaCommand.Execute(null);
            });

            MessagingCenter.Subscribe<TopHeaderBar>(this, PopUpMessages.PopUpAllegatoPrincipale.ToString(), (sender) =>
            {
                Modello.AllegatoPrincipaleCommand.Execute(null);
            });

            MessagingCenter.Subscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.AggiungiAllegato.ToString(), (sender, e) =>
            {
                Modello.AggiungiAllegatoCommand.Execute(e);
            });

            MessagingCenter.Subscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.ListaAllegati.ToString(), (sender, e) =>
            {
                Modello.ListaAllegatiCommand.Execute(e);
            });

            MessagingCenter.Subscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.AggiungiNC.ToString(), (sender, e) =>
            {
                Modello.AggiungiNCCommand.Execute(e);
            });

            MessagingCenter.Subscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.ListaNC.ToString(), (sender, e) =>
            {
                Modello.ListaNCCommand.Execute(e);
            });

            MessagingCenter.Subscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.AzzeraRisposta.ToString(), (sender, e) =>
            {
                Modello.AzzeraRispostaCommand.Execute(e);
            });

            MessagingCenter.Subscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.Help.ToString(), (sender, e) =>
            {
                Modello.HelpCommand.Execute(e);
            });

            MessagingCenter.Subscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.CancellaDomanda.ToString(), (sender, e) =>
            {
                Modello.CancellaDomandaCommand.Execute(e);
            });
        }

        protected override void OnDisappearing()
        {
            base.OnDisappearing();

            MessagingCenter.Unsubscribe<QuestionarioViewModel, AlertModel>(this, Messages.DisplayAlertMessage.ToString());
            MessagingCenter.Unsubscribe<QuestionarioViewModel>(this, Messages.HomePage.ToString());
            MessagingCenter.Unsubscribe<QuestionarioViewModel, string>(this, Messages.ClosedFilterMessage.ToString());

            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpOffline.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpSync.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpCompleta.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpListaLavori.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpConvalida.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpStampa.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpNotificaLista.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpDettaglio.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpRispostaMultipla.ToString());
            MessagingCenter.Unsubscribe<TopHeaderBar>(this, PopUpMessages.PopUpAllegatoPrincipale.ToString());

            MessagingCenter.Unsubscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.AggiungiAllegato.ToString());
            MessagingCenter.Unsubscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.ListaAllegati.ToString());
            MessagingCenter.Unsubscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.AggiungiNC.ToString());
            MessagingCenter.Unsubscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.ListaNC.ToString());
            MessagingCenter.Unsubscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.AzzeraRisposta.ToString());
            MessagingCenter.Unsubscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.Help.ToString());
            MessagingCenter.Unsubscribe<ContextMenuView, EventArgs>(this, ContextMenuMessages.CancellaDomanda.ToString());
        }
    }
}