﻿using AuditManager.Shared.Attributes;
using Newtonsoft.Json;
using SQLite;
using System;
using System.Net;

namespace AuditManager.Models.Tables
{
    [Table("AuditAllegatiRisposta"), TableReplication(Replicate = true)]
    public class AuditAllegatiRisposta : AuditBaseTable
    {
        [NotNull]
        public Guid IdAllegato { get; set; }
        [NotNull]
        public int IdRisposta { get; set; }
        public Guid? IdRispostaDettaglio { get; set; }
        [NotNull]
        public int IdStatoNcCreazione { get; set; }
        [NotNull]
        public string TipoAllegato { get; set; }
        public string PathAllegato { get; set; }
        
        private string _NomeAllegato;
        public string NomeAllegato
        {
            get { return _NomeAllegato; }
            set { _NomeAllegato = WebUtility.HtmlDecode(value); }
        }

        public DateTime DataAllegato { get; set; }
        public Guid? IdAllegatoDupl { get; set; }
        public Guid? IdAllegatoBackend { get; set; }

        [JsonIgnore, Ignore]
        public string Immagine { get; set; }
        [JsonIgnore, Ignore]
        public bool Scaricato { get; set; }

        public AuditAllegatiRisposta() : base()
        {
            IdAllegato = Guid.NewGuid();
            DataAllegato = DateTime.Now;

            Immagine = GetImmagine(TipoAllegato);
            Scaricato = GetScaricato(PathAllegato);

            if (IdAllegato != null)
            {
                IdAllegatoBackend = IdAllegato;
                IdAllegatoDupl = IdAllegato;
            }
        }

        private bool GetScaricato(string pathAllegato)
        {
            bool retValue = true;

            if (pathAllegato.IndexOf("http") > 0)
                retValue = false;

            return retValue;
        }

        private string GetImmagine(string tipoAllegato)
        {
            string retValue = "";

            switch (tipoAllegato)
            {
                case "VIDEO":
                    retValue = "file_video.png";
                    break;
                case "AUDIO":
                    retValue = "file_audio.png";
                    break;
                case "PHOTO":
                    retValue = "file_photo.png";
                    break;
                case "PHOTO_RIT":
                    retValue = "file_photo_rit.png";
                    break;
                case "PAINT":
                    retValue = "file_paint.png";
                    break;
                case "ROLLPITCH":
                    retValue = "file_photo.png";
                    break;
                case "ALTRO":
                default:
                    retValue = "file_altro.png";
                    break;
            }

            return retValue;
        }
    }
}
