﻿using AuditManager.Shared.Attributes;
using AuditManager.Shared.Constants;
using Newtonsoft.Json;
using SQLite;
using System.Net;

namespace AuditManager.Models.Tables
{
    [Table("AuditPersone"), TableReplication(Replicate = true)]
    public class AuditPersone : BaseTable
    {
        [NotNull]
        public int IdPersona { get; set; }
        [NotNull]
        public int IdAudit { get; set; }

        private string _Nome;
        public string Nome
        {
            get { return _Nome; }
            set { _Nome = WebUtility.HtmlDecode(value); }
        }

        private string _Cognome;
        public string Cognome
        {
            get { return _Cognome; }
            set { _Cognome = WebUtility.HtmlDecode(value); }
        }

        public string Telefono { get; set; }
        public string Mail { get; set; }

        private string immagine;
        public string Immagine
        {
            get { return immagine; }
            set
            {
                immagine = value;
                OnPropertyChanged(nameof(Immagine));
            }
        }

        private string firma;
        public string Firma
        {
            get { return firma; }
            set
            {
                firma = value;
                OnPropertyChanged(nameof(Firma));
            }
        }

        public string Messaggio { get; set; }

        [JsonIgnore, Ignore]
        public bool MailVisibile { get; set; }
        [JsonIgnore, Ignore]
        public bool TelefonoVisibile { get; set; }
        [JsonIgnore, Ignore]
        public bool MessaggioVisibile { get; set; }
        [JsonIgnore, Ignore]
        public string NomeCognome { get; set; }


        public AuditPersone() : base()
        {
            Immagine = Constants.ImmaginePlaceHolderUtenti;
            NomeCognome = $"{Nome} {Cognome}";

            MailVisibile = IsVisible(Mail, true);
            TelefonoVisibile = IsVisible(Telefono, false);
            MessaggioVisibile = IsVisible(Messaggio, true);
        }

        public bool IsVisible(string parametro, bool checkRete)
        {
            bool retValue = true;

            if (string.IsNullOrEmpty(parametro))
                retValue = false;
            else
            {
                if (checkRete)
                    retValue = false;
            }

            return retValue;
        }
    }
}
