﻿using System.Linq;
using System.Security.Cryptography;
using System.Text;

namespace AuditManager.Shared.Extensions
{
    public static class StringExtensions
    {
        public static string ToTitleCase(this string s)
        {
            var upperCase = s.ToUpper();
            var words = upperCase.Split(' ');

            var minorWords = new string[] { "ON", "IN", "AT", "OFF", "WITH", "TO", "AS", "BY", "THE", "A", "OTHER", "ANOTHER", "AND", "BUT", "ALSO", "ELSE", "FOR", "IF" };
            var acronyms = new string[] { "UK", "USA", "US", "BBC", "TV" };

            if (acronyms.Contains(words[0]))
                words[0] = words[0].ToUpper();
            else
                words[0] = words[0].ToPascalCase();

            for (int i = 0; i < words.Length; i++)
            {
                if (minorWords.Contains(words[i]))
                    words[i] = words[i].ToLower();
                else if (acronyms.Contains(words[i]))
                    words[i] = words[i].ToUpper();
                else
                    words[i] = words[i].ToPascalCase();
            }

            return string.Join(" ", words);
        }

        public static string ToPascalCase(this string s)
        {
            if (!string.IsNullOrEmpty(s))
                return s.Substring(0, 1).ToUpper() + s.Substring(1).ToLower();
            else
                return string.Empty;
        }

        public static string ReplaceWithNullControl(this string s, string oldValue, string newValue)
        {
            if (string.IsNullOrEmpty(s))
                return s;
            else
            {
                StringBuilder b = new StringBuilder(s);
                b.Replace(oldValue, newValue);
                return b.ToString();
            }
        }

        public static string ReplaceNewlines(this string s, string replaceWith)
        {
            return s.Replace("\r\n", replaceWith).Replace("\n\r", replaceWith).Replace("\n", replaceWith).Replace("\r", replaceWith);
        }

        public static string Sha256(this string s)
        {
            SHA256Managed crypt = new SHA256Managed();
            StringBuilder hash = new StringBuilder();

            byte[] crypto = crypt.ComputeHash(Encoding.UTF8.GetBytes(s));

            foreach (byte theByte in crypto)
            {
                hash.Append(theByte.ToString("x2"));
            }

            return hash.ToString();
        }
    }
}
